SystemOrganization addCategory: #'FastCGI-Pool'!
SystemOrganization addCategory: #FastCGI!
SystemOrganization addCategory: #'FastCGI-Tests'!
SystemOrganization addCategory: #'FastCGI-Protocol'!

SharedPool subclass: #FCGIConstants
	instanceVariableNames: ''
	classVariableNames: 'FCGIAbortRequest FCGIAuthorizer FCGIBeginRequest FCGICantMpxConn FCGIConstants FCGIData FCGIEndRequest FCGIFilter FCGIGetValues FCGIGetValuesResult FCGIHeaderLen FCGIKeepConn FCGIListensockFileno FCGIMaxtype FCGINullRequestId FCGIOverloaded FCGIParams FCGIRequestComplete FCGIResponder FCGIStderr FCGIStdin FCGIStdout FCGIUnknownRole FCGIUnknownType FCGIVersion1'
	poolDictionaries: ''
	category: 'FastCGI-Pool'!

!FCGIConstants class methodsFor: 'initialization' stamp: 'AndreyLarionov 10/7/2009 03:42'!
initialize
	"FCGIConstants initialize"
	FCGIAbortRequest := 2.
	FCGIAuthorizer := 2.
	FCGIBeginRequest := 1.
	FCGICantMpxConn := 1.
	FCGIData := 8.
	FCGIEndRequest := 3.
	FCGIFilter := 3.
	FCGIGetValues := 9.
	FCGIGetValuesResult := 10.
	FCGIHeaderLen := 8.
	FCGIKeepConn := 1.
	FCGIListensockFileno := 0.
	FCGIMaxtype := 11.
	FCGINullRequestId := 0.
	FCGIOverloaded := 2.
	FCGIParams := 4.
	FCGIRequestComplete := 0.
	FCGIResponder := 1.
	FCGIStderr := 7.
	FCGIStdin := 5.
	FCGIStdout := 6.
	FCGIUnknownRole := 3.
	FCGIUnknownType := 11.
	FCGIVersion1 := 1.! !

Error subclass: #FSError
	instanceVariableNames: 'protocolStatus'
	classVariableNames: ''
	poolDictionaries: 'FCGIConstants'
	category: 'FastCGI-Protocol'!

!FSError class methodsFor: 'instance creation' stamp: 'DataCurator 04/28/2007 12:01'!
signalCantMultiplex
	^self new
		protocolStatus: FCGICantMpxConn;
		signal: 'Cant Multiplex'! !

!FSError class methodsFor: 'instance creation' stamp: 'DataCurator 04/28/2007 12:01'!
signalOverloaded
	^self new
		protocolStatus: FCGIOverloaded;
		signal: 'Overloaded'! !

!FSError class methodsFor: 'instance creation' stamp: 'DataCurator 04/28/2007 12:01'!
signalUnknownRole
	^self new
		protocolStatus: FCGIUnknownRole ;
		signal: 'Unknown Role'! !

!FSError methodsFor: 'accessing' stamp: 'DataCurator 04/28/2007 12:01'!
protocolStatus
	^protocolStatus! !

!FSError methodsFor: 'accessing' stamp: 'DataCurator 04/28/2007 12:01'!
protocolStatus: anInteger
	protocolStatus := anInteger! !

Object subclass: #FSConnection
	instanceVariableNames: 'server stream activeId role'
	classVariableNames: ''
	poolDictionaries: 'FCGIConstants'
	category: 'FastCGI-Protocol'!

!FSConnection class methodsFor: 'other' stamp: 'AndreyLarionov 10/8/2009 02:46'!
socket: aSocket
	^self new socket: aSocket! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
activeId
	^activeId! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
activeIdOrManagement
	^activeId isNil ifTrue: [0] ifFalse: [activeId]! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
assert: aBlock

	aBlock value ifFalse: [self error: 'Assertion failed'].! !

!FSConnection methodsFor: 'other' stamp: 'dkh 07/27/2007 13:27'!
assertActiveRequest: aFSRecord
	"Private - If we have an activeId, assert the recordUnit is the same requestId. 
	-Note: Spec says to ignore if not a beginRequest. If this assert signals it is within the Spec to loosen it
	--(I have put a stricter assert in place because I think it would be unusual)"

	(activeId notNil and: [activeId ~= aFSRecord requestId]) 
		ifTrue: [FSError signalCantMultiplex]! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
closeConnection
	"Private - Close the connection
	We are outside #serve's error handling ... this must not signal any errors"

	[stream close] on: Error do: [:e |].
	stream := nil.
	self server removeConnection: self! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
flush
	stream flush! !

!FSConnection methodsFor: 'other' stamp: 'AndreyLarionov 10/8/2009 04:38'!
handleError: anException
	"Private - handle the application error by writing the anException description to stderr.
	We are outside #serve's error handling, if we get an error (reporting the error!!) notify the FSServer"

	[
		| arguments data record |
		"Handle different argument arrangements (probably introduced in 2.3.2)"
		data := anException messageText.
		record := FSRecordSeries 
			stderrId: self activeIdOrManagement
			data: data.
		self writeRecord: record.
	] on: Error do: [:e | 
		self notifyUnreportableError: anException description.
	].! !

!FSConnection methodsFor: 'other' stamp: 'dkh 07/27/2007 13:27'!
handleFSError: anException
	"Private - handle the protocol error by sending back an endRequest record.
	My reading of the spec is that appStatus needs to be 0 otherwise the server will think we have exited.
	We are outside #serve's error handling, if we get an error (reporting the error!!) notify the FSServer" 

	[self server notifyFSError: anException.
	self writeRecord: (
		FSRecordStruct 
			endRequestId: self activeIdOrManagement
			protocolStatus: anException protocolStatus
			appStatus: 0)] on: Error do: [ :e | self notifyUnreportableError: anException description]! !

!FSConnection methodsFor: 'other' stamp: 'dkh 07/27/2007 13:27'!
handleManagement: aRecordUnit
	"Private - SPEC 3.3 ... A management record contains information that is not specific to any Web server request, such as information about the protocol capabilities of the application .... An application may receive a FCGI_GET_VALUES record at any time. The application's response should not involve the application proper but only the FastCGI library"

	| answerPairs |
	aRecordUnit type = FCGIGetValues ifFalse: [^self handleUnknownType: aRecordUnit type].
	answerPairs := self resultsOfQueryPairs: aRecordUnit asPairs.
	self writeRecord: (FSRecordStruct getValuesResultPairs: answerPairs)! !

!FSConnection methodsFor: 'other' stamp: 'dkh 07/27/2007 13:27'!
handleUnknownType: aTypeNum
	"Private - SPEC 4.2 ... When an application receives a management record whose type T it does not understand, the application responds with {FCGI_UNKNOWN_TYPE, 0, {T}}. "

	self writeRecord: (FSRecordStruct unknownType: aTypeNum)! !

!FSConnection methodsFor: 'other' stamp: 'dkh 07/27/2007 13:27'!
makeActive
	"Private - SPEC 3.3 A request ID R becomes active when the application receives a record {FCGI_BEGIN_REQUEST, R, ...} and becomes inactive when the application sends a record {FCGI_END_REQUEST, R, ...} to the Web server. 
While a request ID R is inactive, the application ignores records with requestId == R, except for FCGI_BEGIN_REQUEST records as just described. "

	| recordUnit |
	[(recordUnit := self nextAppRecord) isBeginRequest] whileFalse: [].
	self setActiveIdFor: recordUnit.
	self setConnectionRoleFor: recordUnit.
	recordUnit wantsKeepConnection ifTrue: [FSError signalCantMultiplex]! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
nextAppRecord
	"Answer the next record unit (either a discrete type RecordStruct or a stream type RecordSeries.
	-The data in this record is intended for the application.
	-Management records that may be mixed into the stream have been handled by the receiver"

	| rec |
	^(rec := self nextAppRecordStruct) isStreamRecord 
		ifTrue: [self readSeriesOf: rec]
		ifFalse: [rec]! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
nextAppRecordStruct
	"Private - Answer the next record struct (either a discrete type RecordStruct or a stream type RecordSeries).
	-If the record is for us (ie a management record for the FastCGI Library), handle it and get the next"

	| recordStruct |
	[(recordStruct := stream nextRecordStruct) isManagement] 
		whileTrue: [self handleManagement: recordStruct].
	self assertActiveRequest: recordStruct.
	^recordStruct! !

!FSConnection methodsFor: 'other' stamp: 'dkh 07/27/2007 13:27'!
nextPutAll: aByteObject
	"Stream to stdout."

	self streamRecord: (FSRecordSeries stdoutId: self activeId data: aByteObject)! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
notifyUnreportableError: aString
	"Private - an error has been signaled in trying to report an error back to the WebServer.
	*We are outside #serve's error handling. 
	Inform the server, If this signals an error ignore it ... we have done all we can ie (an error trying to report an error about reporting an error ... something is seriously wrong at this point)" 

	[self server notifyUnreportableError: aString]
		on: Error
		do: [ :e | ]! !

!FSConnection methodsFor: 'other' stamp: 'dkh 07/27/2007 13:27'!
readSeriesOf: aStreamRecord
	^FSRecordSeries forFirstRecord: aStreamRecord streaming: [self nextAppRecordStruct]! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
resultsOfQueryPairs: aCollOfPairs
	"SPEC4.1 ... only three are defined ...Spec was written in 96, if it ever gets updated, deal with it then"

	| dict |
	dict := Dictionary new.
	dict 
		at: 'FCGI_MAX_CONNS' put: [self server maximumConnections printString];
		at: 'FCGI_MAX_REQS' put: [self server maximumRequests printString];	
		at: 'FCGI_MPXS_CONNS' put: ['0'].
	 aCollOfPairs do: [ :each | 
		(dict includesKey: each name) ifTrue: [
			each value: (dict at: each name) value]].
	^aCollOfPairs select: [ :each | each value notEmpty]! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
role
	^role! !

!FSConnection methodsFor: 'other' stamp: 'AndreyLarionov 10/7/2009 03:35'!
roleClassFor: aRoleNum
	| dict |
	(dict := IdentityDictionary new)
		at: FCGIResponder put: FSResponderRole;
		at: FCGIAuthorizer put: FSAuthorizerRole.
	^dict at: aRoleNum ifAbsent: [FSError signalUnknownRole]! !

!FSConnection methodsFor: 'other' stamp: 'AndreyLarionov 10/8/2009 04:10'!
safeServe
	"Private - See #unsafeServe comment.
	-Handle both protocol and application errors"

	[
		[
			[
				self unsafeServe.
			] on: FSError do: [ :e | 
				self handleFSError: e.
			].
		] on: Error do: [ :e | 
			self handleError: e.
		].
	] ensure: [
		self closeConnection.
	].! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
serve
	"Switch method commenting to either serve safely (with exception handling) or unsafe (no handling)"

	self safeServe.! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
server
	"Answer a FSSocketServer"

	^server! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
server: aFSServer
	server := aFSServer! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
setActiveIdFor: recordUnit
	self assert: [activeId isNil].
	activeId := recordUnit requestId.
	self assert: [activeId > 0]! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
setConnectionRoleFor: aFSRecordStruct
	role := (self roleClassFor: aFSRecordStruct role) onConnection: self! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
socket: aSocket 
	"Public - Helper method. 
	-Set the stream constructed from the Socket. The stream is responsible for closing the socket when sent #close
	-The receiver will handle all errors"
 
	self stream: (self streamFrom: aSocket).! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
stream: aRecordStream
	stream := aRecordStream! !

!FSConnection methodsFor: 'other' stamp: 'AndreyLarionov 10/8/2009 03:27'!
streamFrom: aSocket 
	"Private - Helper method. Answer a FSGsSocket for the socket.
	This Stream needs to 
	-Respond to -#nextRecordStruct, #close, #flush.
	-A FSRecord must be able to write itself to the stream.
	-When sent close, it is responsible for closing any socket(s) it is using"
	^ FSSocket on: aSocket! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
streamRecord: aFSRecord
	"Private - SW Experimental
	-Write all but the last record of a Series"

	self assert: [aFSRecord isWSToApp not].
	aFSRecord streamToStream: stream.
	"stream flush. "! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
unsafeServe
	"Private 
		-Check with the FSSocketServer that we should accept the connection.
		-Make the connection active, pass the responsibility of dealing with the connection to the role, then close the connection."

	self server assertAcceptConnection: self.
	self makeActive.
	role handleConnection.! !

!FSConnection methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
writeRecord: aFSRecord
	self assert: [aFSRecord isWSToApp not].
	aFSRecord writeToStream: stream.
	self flush.! !

Object subclass: #FSFCGIStream
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Protocol'!

!FSFCGIStream class methodsFor: 'other' stamp: 'dkh 07/27/2007 13:27'!
convertToByteArray: aFSRecord
	"Test helper method.
	-A ReadWriteByteStream (or at least WriteByteStream) with the protocol of this class, is used to create a ByteArray of the aFSRecord.
	See also #testStreamOn:"

	| stream swazooStream |
	swazooStream := FSStream
		readStream: (ReadStream on: #() asByteArray) 
		writeStream: (WriteStream on: ByteArray new).

	stream := self new stream: swazooStream.
	aFSRecord writeToStream: stream.
	^swazooStream writeStream contents! !

!FSFCGIStream class methodsFor: 'other' stamp: 'dkh 07/27/2007 13:27'!
testStreamOn: aByteArray
	"SW Test helper method.
	Answer a Stream which has the same protocol (or is the same) as  FSFCGIStream or FasTalk's FCGIExternalStream, but on a ReadWriteByteStream.
	-aByteArray is set (and reset) into the readStream."

	| swazooStream |
	swazooStream := FSStream
		readStream: (ReadStream on: aByteArray asByteArray) 
		writeStream: (WriteStream on: ByteArray new).
	^self new stream: swazooStream.! !

!FSFCGIStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
close

	stream close.
	stream := nil.! !

!FSFCGIStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
flush
	^stream flush! !

!FSFCGIStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
next
	"Answer the next byte.
	(If streaming on a socket, block if not available, and signal error if socket not valid)"

	^stream nextByte! !

!FSFCGIStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
next2
	"Answer a ByteArray.
	(If streaming on a socket, block if not available, and signal error if socket not valid)"

	| b1 b0 |
	b1 := self next.
	b0 := self next.
	^b1 * 256 + b0! !

!FSFCGIStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
next: aCount 
	"Answer a ByteArray.
	(If streaming on a socket, block if not available, and signal error if socket not valid)"

	^stream nextBytes: aCount ! !

!FSFCGIStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
nextPut2: aWord
	"(If streaming on a socket signal error if socket not valid)"

	^self 
		nextPut: aWord // 256; 
		nextPut: aWord \\ 256! !

!FSFCGIStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
nextPut: aByte
	"(If streaming on a socket signal error if socket not valid)"

	stream nextPut: aByte! !

!FSFCGIStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
nextPutAll: aByteArray
	"(If streaming on a socket signal error if socket not valid)"

	aByteArray isEmpty ifFalse: [stream nextPutAll: aByteArray]! !

!FSFCGIStream methodsFor: 'other' stamp: 'dkh 07/27/2007 13:27'!
nextRecordStruct
	"Answer the next recordStruct.
	-At this point we do not know whether it is a discrete record, or part of a series
	(If streaming on a socket, block if not available, and signal error if socket not valid)"

	^FSRecordStruct readFromStream: self! !

!FSFCGIStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
stream
	^stream! !

!FSFCGIStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
stream: aReadWriteByteStream
	stream := aReadWriteByteStream! !

Object subclass: #FSLog
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Protocol'!

FSLog class
	instanceVariableNames: 'path'!

FSLog class
	instanceVariableNames: 'path'!

!FSLog class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
disableLog

	path := nil.! !

!FSLog class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
enableLog

	path := '/export/hamburg10/users/jfoster/FastCGI.log'.! !

!FSLog class methodsFor: 'testing' stamp: 'dkh 06/27/2007 11:10'!
enabled

	^path ~~ nil! !

!FSLog class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
log: aString

	| fileStream |
	path isNil ifTrue: [^self].
	GsFile gciLogServer: aString. 
true ifTrue: [^self ].
	fileStream := GsFile openAppendOnServer: path.
	fileStream isNil ifTrue: [^self].
	fileStream 
		cr; 
		nextPutAll: Time now printString; 
		space; 
		nextPutAll: aString;
		close;
		yourself.! !

Object subclass: #FSNameValuePair
	instanceVariableNames: 'name value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Protocol'!

!FSNameValuePair class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
byteAt: index of: anInteger
	"SW:Utility method to aide porting
	Any suggestions welcomed!!"

	"Gss byteAt: is Equivalent to:
		^(self abs bitShift: 1 - index * 8) bitAnd: 16rFF"

	^(anInteger abs bitShift: 1 - index * 8) bitAnd: 16rFF! !

!FSNameValuePair class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
fourBytesFor: aNum
	"SW:Utility method to aide porting
	Any suggestions welcomed!!"

	| byteArray |
	byteArray := ByteArray new: 4.
	1 to: 4 do: [ :i | byteArray at: i put: (self byteAt: 5 - i of: aNum)].
	^byteArray! !

!FSNameValuePair class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
fromAssociation: anAssociation
	^self 
		name: anAssociation key
		value: anAssociation value! !

!FSNameValuePair class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
name: aNameString value: aValueString
	^self new name: aNameString value: aValueString! !

!FSNameValuePair class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
next1Or4From: aStream
	| ch |
	ch := aStream next.
	ch > 127 ifTrue: [ch := ch - 128 * 16777216 + (aStream next * 65536) + (aStream next * 256) + aStream next].
	^ch! !

!FSNameValuePair class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
oneOrFourBytesFor: length
	^length  <= 127
		ifTrue: [ByteArray with: length]
		ifFalse: [| byteArray |
			byteArray := self fourBytesFor: length.
			byteArray at: 1 put: (byteArray first bitOr: 16r80).
			byteArray]! !

!FSNameValuePair class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
readFromStream: aReadByteStream
	^self new readFromStream: aReadByteStream! !

!FSNameValuePair methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
asByteArray
	| stream |
	stream := WriteStream on: ByteArray new.
	self writeToStream: stream.
	^stream contents! !

!FSNameValuePair methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
name
	^name! !

!FSNameValuePair methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
name: aNameString value: aValueString
	name := aNameString.
	value := aValueString.! !

!FSNameValuePair methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
readFromStream: aStream
	| nameLength valueLength |
	nameLength := self class next1Or4From: aStream.
	valueLength := self class next1Or4From: aStream.
	name := (aStream next: nameLength) asString.
	value := (aStream next: valueLength) asString.! !

!FSNameValuePair methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
value
	^value! !

!FSNameValuePair methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
value: aString 
	value := aString ! !

!FSNameValuePair methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
writeToStream: aStream
	aStream 
		nextPutAll: (self class oneOrFourBytesFor: name size);
		nextPutAll: (self class oneOrFourBytesFor: value size);
		nextPutAll: name asByteArray;
		nextPutAll: value asByteArray! !

Object subclass: #FSRecordAbstract
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: 'FCGIConstants'
	category: 'FastCGI-Protocol'!

!FSRecordAbstract class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
assert: aBlock

	aBlock value ifFalse: [self error: 'Assertion failed'].! !

!FSRecordAbstract class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
managementTypes 
	"Private - SPEC: http://www.fastcgi.com/devkit/doc/fcgi-spec.html#SA
	Record types that are intended for the FCGILibrary, not the application"
	
	"^#(FCGI_GET_VALUES  FCGI_GET_VALUES_RESULT  FCGI_UNKNOWN_TYPE)"
	^#(9 10 11)! !

!FSRecordAbstract class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
streamTypes 
	"Private - SPEC: http://www.fastcgi.com/devkit/doc/fcgi-spec.html#SA
	Record types that form part of a stream"
	
	"^#(FCGI_PARAMS FCGI_STDIN FCGI_DATA FCGI_STDOUT FCGI_STDERR)"
	^#(4 5 8 6 7)! !

!FSRecordAbstract class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
type: aTypeNum requestId: anID data: aBytesObject
	^self new 
		setType: aTypeNum
		requestId: anID
		data: aBytesObject! !

!FSRecordAbstract class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
wsToAppTypes 
	"Private - SPEC: http://www.fastcgi.com/devkit/doc/fcgi-spec.html#SA
	Record types that can be sent to us"
	
	"^#(FCGI_GET_VALUES  FCGI_BEGIN_REQUEST  FCGI_ABORT_REQUEST  FCGI_PARAMS  FCGI_STDIN  FCGI_DATA)"
	^#(9 1 2 4 5 8)! !

!FSRecordAbstract methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
= aRecord
	"SW: Test helper method. Not efficient, but sufficient"

	^aRecord species == self species and: [
		self asByteArray = aRecord asByteArray]! !

!FSRecordAbstract methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
asByteArray
	"SW: Test Helper method"

	^FSFCGIStream convertToByteArray: self! !

!FSRecordAbstract methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
asPairs
	| stream pairs |
	stream := self data readStream.
	pairs := OrderedCollection new.
	[stream atEnd] whileFalse: [ 
		pairs add: (FSNameValuePair readFromStream: stream)].
	^pairs! !

!FSRecordAbstract methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
assert: aBlock

	aBlock value ifFalse: [self error: 'Assertion failed'].! !

!FSRecordAbstract methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
assertValid
	self subclassResponsibility! !

!FSRecordAbstract methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
data
	self subclassResponsibility! !

!FSRecordAbstract methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
hash
	"SW: Test helper method. Not efficient, but sufficient"

	^self asByteArray hash! !

!FSRecordAbstract methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
initialize! !

!FSRecordAbstract methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
isBeginRequest
	^self type = FCGIBeginRequest! !

!FSRecordAbstract methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
isManagement
	"SPEC: http://www.fastcgi.com/devkit/doc/fcgi-spec.html#SA"
	
	"^self class managementTypes includes: self type"
	^self requestId = 0! !

!FSRecordAbstract methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
isStreamRecord
	"SPEC: http://www.fastcgi.com/devkit/doc/fcgi-spec.html#SA"
	
	^self class streamTypes includes: self type! !

!FSRecordAbstract methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
isWSToApp
	"SPEC: http://www.fastcgi.com/devkit/doc/fcgi-spec.html#SA"
	
	^self class wsToAppTypes includes: self type! !

!FSRecordAbstract methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
requestId
	self subclassResponsibility! !

!FSRecordAbstract methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
type
	self subclassResponsibility! !

!FSRecordAbstract methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
writeToStream: aStream 
	self subclassResponsibility! !

FSRecordAbstract subclass: #FSRecordSeries
	instanceVariableNames: 'records'
	classVariableNames: ''
	poolDictionaries: 'FCGIConstants'
	category: 'FastCGI-Protocol'!

!FSRecordSeries class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
forFirstRecord: aRecordStruct 
	^self new 
		addFirstRecord: aRecordStruct ;
		yourself! !

!FSRecordSeries class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
forFirstRecord: aRecordStruct streaming: operation
	| nextRecord series |
	series := self  forFirstRecord: aRecordStruct .
	nextRecord := aRecordStruct .
	[nextRecord isLastInSeries] 
		whileFalse: [series addNextRecord: (nextRecord := operation value)].
	^series! !

!FSRecordSeries class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
new
	^super new initialize! !

!FSRecordSeries class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
paramsId: anId pairs: aCollOfPairs
	"Private - SW: Test helper method"

	^self type: FCGIParams requestId: anId pairs: aCollOfPairs! !

!FSRecordSeries class methodsFor: 'other' stamp: ''!
readFromStream: aStream
	| record |
	self assert: [(record := aStream nextRecordStruct) isStreamRecord].
	^self forFirstRecord: record streaming: [aStream nextRecordStruct]! !

!FSRecordSeries class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
stderrId: anId data: aString
	^self type: FCGIStderr requestId: anId data: aString! !

!FSRecordSeries class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
stdinId: anId data: aString
	"Private - SW: Test helper method"

	^self type: FCGIStdin requestId: anId data: aString! !

!FSRecordSeries class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
stdoutId: anId data: aString
	^self type: FCGIStdout requestId: anId data: aString! !

!FSRecordSeries class methodsFor: 'other' stamp: 'dkh 07/27/2007 13:27'!
type: aTypeNum requestId: anId pairs: aCollOfPairs
	"Private - SW: do the common practice and code each pair as a record.
	Duplication with instance>>#setType:requestId:data:"

	| instance |
	instance := self new.
	aCollOfPairs do: [ :each |
		instance addRecord: (FSRecordStruct 
			type: aTypeNum
			requestId: anId
			data: each asByteArray)].
	instance addRecord: (FSRecordStruct 
				type: aTypeNum
				requestId: anId
				data: '').
	^instance


	! !

!FSRecordSeries methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
addFirstRecord: aRecordStruct
	self assert: [aRecordStruct isStreamRecord].
	self assert: [records isEmpty].
	^records add: aRecordStruct! !

!FSRecordSeries methodsFor: 'other' stamp: 'dkh 07/27/2007 13:27'!
addNextRecord: aRecordStruct 
	"Private - SW: Note on the type assert: My reading of the Spec is that at the record layer, it is possible to interleave stream records of different types. However at the role/connection level for a FCGIApplication, this would be an error. If this assert signals, please contact me"

	self assert: [aRecordStruct isStreamRecord].
	self assert: [aRecordStruct type = self type].
	aRecordStruct requestId = self requestId ifFalse: [FSError signalCantMultiplex].
	^records add: aRecordStruct ! !

!FSRecordSeries methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
addRecord: aRecordStruct
	^records isEmpty 
		ifTrue: [self addFirstRecord: aRecordStruct]
		ifFalse: [self addNextRecord: aRecordStruct]! !

!FSRecordSeries methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
asParams
	"Answer a dictionary of converted FCGI name value pairs"

	| params |
	params := Dictionary new.
	self asPairs do: [ :each | params at: each name put: each value].
	^params! !

!FSRecordSeries methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
assertValid
	"Private - SPEC 3.3 (is) .. a series of zero or more non-empty records ... followed by an empty record"

	self assert: [records notEmpty].
	self assert: [self lastRecord isLastInSeries]! !

!FSRecordSeries methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
data
	"Answer the data of the record(series) as a ByteArray"

	| stream |
	stream := WriteStream on: ByteArray new.
	records do: [ :each | stream nextPutAll: each data].
	^stream contents! !

!FSRecordSeries methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
firstRecord
	^records first! !

!FSRecordSeries methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
initialize
	super initialize.
	records := OrderedCollection new! !

!FSRecordSeries methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
lastRecord
	^records last! !

!FSRecordSeries methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
records
	^records! !

!FSRecordSeries methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
requestId
	^self firstRecord requestId! !

!FSRecordSeries methodsFor: 'other' stamp: 'dkh 04/09/2008 16:54'!
setType: aTypeNum requestId: anId data: aByteArray 

	| stream |
	aByteArray size < 65535
		ifTrue: [
			self addRecord: (FSRecordStruct 
				type: aTypeNum
				requestId: anId
				data: aByteArray).
		]
		ifFalse: [ | pos limit |
			pos := 1.
			limit := aByteArray size.
			[ pos <= limit ] whileTrue: [
				| count |
				count := limit - pos + 1 min: 65535.
				self addRecord: (FSRecordStruct 
					type: aTypeNum
					requestId: anId
					data: (aByteArray copyFrom: pos to: pos + count - 1)).
				pos := pos + count
			].
		].
	self addRecord: (FSRecordStruct 
		type: aTypeNum
		requestId: anId
		data: '')! !

!FSRecordSeries methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
streamToStream: aFSFCGIGsStream 
	"Write all but the last record
	This can be used to send part of a bigger series"

	self assert: [self lastRecord isLastInSeries].
	records from: 1 to: records size -1 do: [ :each | each writeToStream: aFSFCGIGsStream ]! !

!FSRecordSeries methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
type
	"Answer the typeNum of the FCGI Record series"

	^self firstRecord type! !

!FSRecordSeries methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
writeToStream: aFSFCGIGsStream 
	self assert: [self lastRecord isLastInSeries].
	records do: [ :each | each writeToStream: aFSFCGIGsStream]! !

FSRecordAbstract subclass: #FSRecordStruct
	instanceVariableNames: 'type requestId contentLength paddingLength data'
	classVariableNames: ''
	poolDictionaries: 'FCGIConstants'
	category: 'FastCGI-Protocol'!

!FSRecordStruct class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
beginRequestId: anId role: anInteger1 flags: anInteger2
	"Private - SW: Test Helper method"
	
	self assert: [anInteger1 < 256].
	self assert: [anInteger2 < 256].
	^self type: FCGIBeginRequest requestId: anId data:  (self dataWith: 0 with: anInteger1  with: anInteger2)
	! !

!FSRecordStruct class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
dataWith: i1 
	"Private - Helper method"

	^self emptyData
		at: 1 put: i1;
		yourself
	! !

!FSRecordStruct class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
dataWith: i1 with: i2  with: i3
	"Private - Helper method"

	^self emptyData
		at: 1 put: i1;
		at: 2 put: i2;
		at: 3 put: i3;
		yourself
	! !

!FSRecordStruct class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
emptyData
	"Private - Helper method"

	^(ByteArray new: 8) atAllPut: 0; yourself.! !

!FSRecordStruct class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
endRequestId: anId protocolStatus:  aNum appStatus: aNum2
	"SW: typedef struct {
	unsigned char appStatusB3;	
	unsigned char appStatusB2;
	unsigned char appStatusB1;
	unsigned char appStatusB0;
	unsigned char protocolStatus;
	unsigned char reserved[3]; } FCGI_EndRequestBody; "

	| data |
	data := self emptyData.
	data replaceFrom: 1 to: 4 with: (FSNameValuePair fourBytesFor: aNum2).
	data at: 5 put: aNum.
	^self type: FCGIEndRequest requestId: anId data:  data	! !

!FSRecordStruct class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
getValuesPairs: aCollOfPairs
	"Private- SW: Test helper method"

	^self type: FCGIGetValues requestId: 0 pairs: aCollOfPairs! !

!FSRecordStruct class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
getValuesResultPairs: aCollOfPairs
	"SPEC 4.1"
 
	^self type: FCGIGetValuesResult requestId: 0 pairs: aCollOfPairs! !

!FSRecordStruct class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
readFromStream: aFCGIExternalStream 
	^self new readFromStream: aFCGIExternalStream! !

!FSRecordStruct class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
type: aTypeNum requestId: anId pairs: aCollOfPairs
	"Private - Answer a discrete record that contains pairs (FCGI_GET_VALUES and FCGI_GET_VALUES_RESULT)
	SPEC4.1  "

	| stream |
	stream := WriteStream on: ByteArray new.
	aCollOfPairs do: [ :each | each writeToStream: stream].
	^self 
		type: aTypeNum 
		requestId: anId
		data: stream contents ! !

!FSRecordStruct class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
unknownType: aTypeNumber
	"SW: FCGI_UNKNOWN_TYPE is a management record
	The contentData component of a FCGI_UNKNOWN_TYPE record has the form: 
        typedef struct {
            unsigned char type;    
            unsigned char reserved[7];
        } FCGI_UnknownTypeBody;"

	self assert: [aTypeNumber isKindOf: Integer].
	^self type: FCGIUnknownType  requestId: 0 data:  (self dataWith: aTypeNumber)
		
	! !

!FSRecordStruct methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
assertValid
	(self isBeginRequest and: [self contentLength ~= 8]) 
		ifTrue: 
			[self error: 'Malformed BeginRequest packet received with content length: ' 
						, self contentLength printString]! !

!FSRecordStruct methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
contentLength
	^contentLength! !

!FSRecordStruct methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
data
	"Answer the data of the record as a ByteArray"

	^data! !

!FSRecordStruct methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
flags
	self assert: [self isBeginRequest].
	^self data at: 3.! !

!FSRecordStruct methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
isLastInSeries
	"Private - SPEC 3.3 ... a series (is) ... followed by an empty record (length == 0) "

	self assert: [self isStreamRecord].
	^self contentLength = 0! !

!FSRecordStruct methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
paddingLength
	paddingLength isNil ifTrue: [paddingLength := 0].
	^paddingLength! !

!FSRecordStruct methodsFor: 'other' stamp: 'DataCurator 05/16/2007 16:15'!
readFromStream: aStream 
	"aStream has the protocol of aFSFCGIGsStream"

	| version |
	(version := aStream next) == FCGIVersion1 ifFalse: [
		self error: 'Incorrect FCGI version. Expected: 1 Received: ' , version printString].
	type := aStream next.
	requestId := aStream next2.
	contentLength := aStream next2.
	paddingLength := aStream next.
	aStream next. "reserved"
	data := (aStream next: self contentLength).
	paddingLength > 0 ifTrue: [aStream next: paddingLength].
	self assertValid! !

!FSRecordStruct methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
requestId
	^requestId! !

!FSRecordStruct methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
role
	self assert: [self isBeginRequest].
	^((self data at: 1) * 256) + (self data at: 2)! !

!FSRecordStruct methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
setType: aSmallInteger requestId: aSmallInteger2 data: aByteObject
	type := aSmallInteger.
	requestId := aSmallInteger2.
	data := aByteObject asByteArray.
	contentLength := data size.
	paddingLength := 0! !

!FSRecordStruct methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
streamToStream: aFSFCGIGsStream 
	self error: 'Not supported for discrete records'! !

!FSRecordStruct methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
type
	^type! !

!FSRecordStruct methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
wantsKeepConnection
	"SPEC 5.1 flags & FCGI_KEEP_CONN: If zero, the application closes the connection after responding to this request. If not zero, the application does not close the connection after responding to this request; the Web server retains responsibility for the connection."

	^(self flags bitAnd: FCGIKeepConn) ~= 0! !

!FSRecordStruct methodsFor: 'other' stamp: 'AndreyLarionov 10/7/2009 03:34'!
writeToStream: aFSFCGIGsStream 
 
  | header | 
  (header := ByteArray new: 8) 
    byteAt:  1 put: FCGIVersion1; 
    byteAt:  2 put: type; 
    unsignedShortAt: 3 put: requestId bigEndian: true;  
    unsignedShortAt: 5 put: contentLength bigEndian: true; 
    byteAt:  7 put: self paddingLength; 
    byteAt:  8 put: 0; 
    yourself. 
  aFSFCGIGsStream 
    nextPutAll: header; 
    nextPutAll: self data; 
    yourself. 
! !

Object subclass: #FSRequestHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Protocol'!

FSRequestHandler subclass: #FSDemoHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Protocol'!

!FSDemoHandler class methodsFor: 'as yet unclassified' stamp: 'AndreyLarionov 10/7/2009 22:16'!
startUp
	"self startUp"

	^(FSSqSocketServer onRequestHandler: self new)
		port: 9001;
		start;
		yourself.! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
answerResponderRole: aFSResponderRole 
	"Note: This class attempts to do many things in order to test a FS setup
	-See superclass comment.
	-If the request contains a query we recognize, return one of the special case 'testing' responses, otherwise return the default response."

	| selector |
	selector := self querySelectorOrNil: aFSResponderRole.
	selector isNil ifTrue: [
		self handleByUri: aFSResponderRole.
	] ifFalse: [
		self perform: selector with: aFSResponderRole.
	].! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
cgiLocation: aResponder
	"Private - respond by setting stdout"

	| stream |
	stream := WriteStream on: String new.
	stream
		nextPutAll: 'Location: /index.html';
		nextPutAll: self crlf;
		nextPutAll: self crlf.
	aResponder stdout: stream contents! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
chunked: aResponder
	"Private - respond by streaming to stdout.
	The presence of the Status header indicates we are returning a 'Non-parsed Header response', not a CGI response.
	See <http://cgi-spec.golux.com/draft-coar-cgi-v11-03-clean.html#7.1>"

	aResponder
		nextPutAll: 'Status: 200 OK';
		cr;
		nextPutAll: 'Content-type: text/html';
		cr;
		nextPutAll: 'Transfer-Encoding: chunked';
		cr;
		cr.
	self 
		putChunk: '<html><body>
<h1>Chunk Transfer encoding test</h1>
<p>This is the first chunk</p>'
		on: aResponder.
	self putChunk: '<p>Chunk2</p>' on: aResponder.
	self putChunk: '<p>Chunk3</p>' on: aResponder.
	self putChunk: '<p>Thanks for playing!!</p></body></html>' on: aResponder.
	self putChunk: '' on: aResponder! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
crlf
	"Private - Answer a crlf string"

	^(String new: 2)
		at: 1 put: Character cr;
		at: 2 put: Character lf;
		yourself.! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
defaultFor: aFSResponderRole 
	"Private - respond by setting stdout."

	| stream |
	stream := self getCGIStream.
	self putStartOn: stream.
	self putDescriptionOn: stream.
	self putInfoOn: stream.
	self putTestsOn: stream.
	self putDataOn: stream for: aFSResponderRole .
	self putEndOn: stream.

	aFSResponderRole stdout: stream contents.! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
delay: aResponder
	"Private - respond by setting stdout.
	The presence of the Status header indicates we are returning a 'Non-parsed Header response', not a CGI response.
	See <http://cgi-spec.golux.com/draft-coar-cgi-v11-03-clean.html#7.1>"

	| stream |
	(Delay forMilliseconds: 200) wait.
	stream := WriteStream on: String new.
	stream
		nextPutAll: 'Status: 200 OK';
		nextPutAll: self crlf;
		nextPutAll: 'Content-type: text/html';
		nextPutAll: self crlf;
		nextPutAll: self crlf;
		nextPutAll: '<html><body>
<h1>Delayed 200 OK</h1>
<p>Thanks for playing!!</p>
</body></html>'.
	aResponder stdout: stream contents! !

!FSDemoHandler methodsFor: 'other' stamp: 'AndreyLarionov 10/8/2009 04:38'!
exception: aResponder
	self error: 'Test exception'! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
getCGIStream
	"Private - Helper method, answer a stream which already contains the standard CGI headers"

	| stream |
	stream := WriteStream on: String new.
	stream
		nextPutAll: 'Content-type: text/html';
		nextPutAll: self crlf;
		nextPutAll: self crlf.
	^stream! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
handleByUri: aFSResponderRole

	^self defaultFor: aFSResponderRole! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
http302: aResponder
	"Private - respond by setting stdout.
	The presence of the Status header indicates we are returning a 'Non-parsed Header response', not a CGI response.
	See <http://cgi-spec.golux.com/draft-coar-cgi-v11-03-clean.html#7.1>"

	| stream |
	stream := WriteStream on: String new.
	stream 
		nextPutAll: 'Status: 302 Found';
		nextPutAll: self crlf;
		nextPutAll: 'Location: /index.html';
		nextPutAll: self crlf;
		nextPutAll: self crlf;
		nextPutAll: 
'<html><head><title>Redirection</title></head>
<body><p>If you are seeing this, the test has probably failed</p></body></html>' .
	aResponder stdout: stream contents.! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
large: aResponder
	"Private - respond by setting stdout"

	| stream |
	stream := self getCGIStream.
	stream 
		nextPutAll: 
'<html><body>
<h3>Large test</h3>
<p>One large string to be sent (Multiple FCGI records)</p>
<p>'.
	7000 timesRepeat: [stream nextPutAll: 'Smalltalk '].
	stream nextPutAll: '</p><p>Thanks for playing!!</p></body></html>'.
	aResponder stdout: stream contents.! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
listStrings: aCollOfStrings on: aStream
	aCollOfStrings do: 
			[:each | 
			aStream
				nextPutAll: '<li>';
				nextPutAll: each;
				nextPutAll: '</li>']! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
paramsAsHTML: aDictionary
	| stream |
	stream := WriteStream on: String new.
	stream nextPutAll: '<table>'.
	aDictionary keys asSortedCollection do: [:k | 
		| v |
		v := aDictionary at: k.
		stream
			nextPutAll: '<tr><td>';
			nextPutAll: k;
			nextPutAll: '</td><td>';
			nextPutAll: v;
			nextPutAll: '</td></tr>';
			cr.
	].
	stream nextPutAll: '</table>'.
	^stream contents! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
putChunk: aString on: stream
	stream
		nextPutAll: (aString size printStringRadix: 16 showRadix: false);
		cr;
		nextPutAll: aString;
		cr! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
putDataOn: stream for: aFSResponderRole 
	stream
		nextPutAll: '<h3>Params:</h3>';
		nextPutAll: (self paramsAsHTML: aFSResponderRole params);
		nextPutAll: '<h3>Stdin:</h3>';
		nextPutAll: aFSResponderRole stdin printString

	! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
putDescriptionOn: stream
	stream 
		nextPutAll: '<h3>Welcome!!</h3>
<h3>Notes:</h3>
<ul>
<li>Version: 030</li>
<li>This is demonstration handler for a <a href="http://www.fastcgi.com">FastCGI</a> Application implementation for GemStone Smalltalk.</li>
<li>HTTPRequests to the FastCGI WebServer (Apache server with mod_fastcgi) are sent to the image,
and the CGI parameters and stdin data can be accessed by the handler instance.</li>
<li>By default, this handler simply echoes the parameters and stdin data it receives to stdout.</li>
<li>By Steve Waring 2002. (Certain methods and classes based on FasTalk by Tomas Vanak)</li>
</ul>'! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
putEndOn: stream
	stream
		nextPutAll: '<hr/>';
		nextPutAll: self thanksString;
		nextPutAll: '</body></html>'.
		
	! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
putInfoOn: stream ! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
putStartOn: stream
	stream nextPutAll: '<html><body><h2>FSDemoHandler - ResponderRole</h2>'! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
putTestsOn: stream
	stream nextPutAll:
'<h3>Tests:</h3>

<p>(add as query to a url, ie ?status200)</p>
<ul>
  <li><a href="test?status200">status200</a> - response with a 200 Status header.</li>
  <li><a href="test?status500">status500</a> - response with a 500 Status header.</li>
  <li><a href="test?stream">stream</a> - streams back a response (Apache may cache).</li>
  <li><a href="test?streamDelay">streamDelay</a> - as above with 1 second delays.</li>
  <li><a href="test?large">large</a> - response spans multiple FastCGI records.</li>
  <li><a href="test?imageState">imageState</a> - information from the image.</li>
  <li><a href="test?exception">exception</a> - to signal an error.</li>
  <li><a href="testing1?cgiLocation">CGI Location</a> - response with CGI Location header /index.html.</li>
  <li><a href="testing2?http302">HTTP 302</a> - response with HTTP Location header /index.html.</li>
  <li><a href="test?chunked">Chunked Transfer encoding</a> - chunked response.</li>
</ul>'! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
querySelectorOrNil: aFSResponderRole
	| querySymbol |
	querySymbol := ((aFSResponderRole params at: 'QUERY_STRING' ifAbsent: [^nil]) , ':') 
				asSymbol.
	^(self querySelectors includes: querySymbol) ifTrue: [querySymbol] ifFalse: []! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
querySelectors
	^#(#exception: #stream: #streamDelay: #large: #status500: #status200: #cgiLocation: #http302: #chunked: #delay:) 
		asOrderedCollection! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
status200: aResponder
	"Private - respond by setting stdout.
	The presence of the Status header indicates we are returning a 'Non-parsed Header response', not a CGI response.
	See <http://cgi-spec.golux.com/draft-coar-cgi-v11-03-clean.html#7.1>"

	| stream |
	stream := WriteStream on: String new.
	stream
		nextPutAll: 'Status: 200 OK';
		nextPutAll: self crlf;
		nextPutAll: 'Content-type: text/html';
		nextPutAll: self crlf;
		nextPutAll: self crlf;
		nextPutAll: '<html><body>
<h1>200 OK</h1>
<p>Thanks for playing!!</p>
</body></html>'.
	aResponder stdout: stream contents! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
status500: aResponder
	"Private - respond by setting stdout.
	The presence of the Status header indicates we are returning a 'Non-parsed Header response', not a CGI response.
	See <http://cgi-spec.golux.com/draft-coar-cgi-v11-03-clean.html#7.1>"

	| stream |
	stream := WriteStream on: String new.
	stream
		nextPutAll: 'Status: 500 Internal Server Error';
		nextPutAll: self crlf;
		nextPutAll: 'Content-type: text/html';
		nextPutAll: self crlf;
		nextPutAll: self crlf;
		nextPutAll: '<html><body>
<h1>500 Internal Server Error</h1>
<p>Thanks for playing!!</p>
</body></html>'.
	aResponder stdout: stream contents! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
stream: aResponder
	"Private - respond by streaming to stdout"

	self stream: aResponder withDelay: nil! !

!FSDemoHandler methodsFor: 'other' stamp: 'AndreyLarionov 10/8/2009 04:26'!
stream: aResponder withDelay: seconds
	"Private - respond by streaming to stdout"

	aResponder 
		nextPutAll: self getCGIStream contents;
		nextPutAll: 
'<html><body>
<h3>Streaming test</h3>
<p>9 "paragraphs" will be streamed.</p>'.
	'Smalltalk' do: [ :each |
		aResponder 
			nextPutAll: '<p>';
			nextPutAll: ((String new: 5000) atAllPut: each; yourself);
			nextPutAll: '</p>'].
	seconds notNil ifTrue: [(Delay forSeconds: seconds) wait].
	aResponder nextPutAll: '<p>Thanks for playing!!</p>'.	
	aResponder nextPutAll: '</body></html>'.! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
streamDelay: aResponder
	"Private - respond by streaming to stdout"

	self stream: aResponder withDelay: 1! !

!FSDemoHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
thanksString
	^'<p><a  href="mailto:swaring@ozemail.com.au">Steve Waring</a></p>'! !

!FSRequestHandler class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
new
	^super new initialize! !

!FSRequestHandler methodsFor: 'other' stamp: 'dkh 07/27/2007 13:27'!
answerAuthorizerRole: aFSAuthorizerRole
	"Answer to the aFSResponderRole. Subclasses should re-implement this method. See #answerResponderRole:
	See SPEC 6.3
	1/ Access  the FCGI_PARAMS stream via #params 
	2/ sends stdout and stderr data in the same manner as a Responder
	See http://www.fastcgi.com/devkit/doc/fcgi-spec.html#S6.3
		http://www.fastcgi.com/mod_fastcgi/docs/mod_fastcgi.html (Apache - FastCGI Specification Compliance )"

	FSError signalUnknownRole! !

!FSRequestHandler methodsFor: 'other' stamp: 'dkh 07/27/2007 13:27'!
answerResponderRole: aFSResponderRole 
	"Answer to the aFSResponderRole. Subclasses should re-implement this method. 
	1/ Applications can (via the aFSResponderRole argument)
		-access #stdin <ByteArray> and #params <Dictionary>
		-set #stderr: <String or ByteArray>s. 
		Either: -set #stdout: <String or ByteArray>s
		Or -stream to stdout using #nextPut:/#nextPutAll:/#nextPutBytes:/ etc.
	2/ The receiver is shared from multiple processes. The role object is created within this process.
	3/ Exceptions are handled and sent back to the WebServer via stderr
	4/ When this method returns, stdout and stderr will be sent back to the WebServer.

	See http://www.fastcgi.com/devkit/doc/fcgi-devel-kit.htm and http://cgi-spec.golux.com/"

	FSError signalUnknownRole! !

!FSRequestHandler methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
initialize! !

Object subclass: #FSRole
	instanceVariableNames: 'connection params stdin stdout stderror'
	classVariableNames: ''
	poolDictionaries: 'FCGIConstants'
	category: 'FastCGI-Protocol'!

FSRole subclass: #FSAuthorizerRole
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: 'FCGIConstants'
	category: 'FastCGI-Protocol'!

!FSAuthorizerRole methodsFor: 'other' stamp: 'dkh 07/27/2007 13:33'!
answer
	"Private - Ask the requestHandler attached to the FSServer to answer to the receiver"

	self requestHandler answerAuthorizerRole: self! !

!FSAuthorizerRole methodsFor: 'other' stamp: 'dkh 07/27/2007 13:33'!
receive
	"Private - 
	1/ The Authorizer application receives CGI/1.1 environment variables over FCGI_PARAMS"

	self readParams.! !

!FSAuthorizerRole methodsFor: 'other' stamp: 'dkh 07/27/2007 13:33'!
roleName
	^'Authorizer'! !

!FSAuthorizerRole methodsFor: 'other' stamp: 'dkh 07/27/2007 13:33'!
send
	"Private - The Authorizer application sends stdout and stderr data in the same manner as a Responder"

	self sendStderr.
	self sendStdout.
	self sendEndRequestSuccess.! !

FSRole subclass: #FSResponderRole
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: 'FCGIConstants'
	category: 'FastCGI-Protocol'!

!FSResponderRole methodsFor: 'other' stamp: 'dkh 07/27/2007 13:33'!
answer
	"Private - Ask the requestHandler attached to the FSServer to answer to the receiver"

	self requestHandler answerResponderRole: self.! !

!FSResponderRole methodsFor: 'other' stamp: 'dkh 07/27/2007 13:33'!
isHttp

	^true.! !

!FSResponderRole methodsFor: 'other' stamp: 'dkh 07/27/2007 13:33'!
receive
	"Private - 
	1/ The Responder application receives CGI/1.1 environment variables over FCGI_PARAMS
	2/ Next the Responder application receives CGI/1.1 stdin data "

	self readParams.
	self readStdin.
	self assertStdinLength.! !

!FSResponderRole methodsFor: '*FastCGI-Seaside' stamp: 'dkh 09/12/2007 09:27'!
remoteAddress

	^self params at: 'REMOTE_ADDR' ifAbsent: [''].! !

!FSResponderRole methodsFor: 'other' stamp: 'dkh 07/27/2007 13:33'!
roleName
	^'Responder'! !

!FSResponderRole methodsFor: 'other' stamp: 'dkh 07/27/2007 13:33'!
send
	"Private - 
	1/ The Responder application sends CGI/1.1 stdout data to the Web server over FCGI_STDOUT, and CGI/1.1 stderr data over FCGI_STDERR. The application sends these concurrently, not one after the other.
	2/ After sending all its stdout and stderr data, the Responder application sends a FCGI_END_REQUEST record"

	self sendStderr.
	self sendStdout.
	self sendEndRequestSuccess.! !

!FSRole class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
onConnection: aFSConnection 
	^self new connection: aFSConnection! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
activeId
	^connection activeId! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
answer
	"Private - Ask the requestHandler attached to the FSServer to answer to the receiver"

	self subclassResponsibility! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
assertStdinLength
	self contentLengthHeader = stdin size ifFalse: [
		self error: 'Incorrect content length. Role: ' , self roleName]
	! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
connection
	"Private - Answer aFSConnection ."

	^connection! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
connection: aFSConnection
	connection := aFSConnection! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
contentLengthHeader
	^(params at: 'CONTENT_LENGTH' ifAbsent: ['0']) asNumber! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
cr
	"This method is part of a puttableStream protocol for stdout"
	
	self crlf.! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
crlf
	"This method is part of a puttableStream protocol for stdout"
	
	self nextPutAll: (ByteArray with: 13 with: 10)
	! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
flush
	"This method is part of a puttableStream protocol for stdout
	-No-op"

	^self connection flush
	! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
handleConnection

	self 
		receive;
		answer;
		send;
		yourself.! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
nextPut: anObject
	"This method is part of a puttableStream protocol for stdout"
	
	self nextPutAll: (ByteArray with: anObject asInteger)
	! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
nextPutAll: aByteObject
	"This method is part of a puttableStream protocol for stdout
	-Note: an empty string will *not* be sent. (The WebServer will take that as an indication that
	we have finished sending stdout. If the stdout string is empty or nil when the server returns from answering the role,
	it will automatically write an empty stdout record for us."
	
	self connection nextPutAll: aByteObject
	! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
nextPutBytes: aByteObject
	"This method is provides FSStream compatibility to the stdout  puttableStream"

	self nextPutAll: aByteObject
	! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
params
	^params! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
readParams
	| recordUnit |
	(recordUnit := self connection nextAppRecord) type = FCGIParams 
		ifFalse: [self error: 'Error expected: FCGIParams, Role: ' , self roleName].
	params := recordUnit asParams! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
readStdin
	| recordUnit |
	(recordUnit := self connection nextAppRecord) type = FCGIStdin 
		ifFalse: [self error: 'Error expected: FCGIStdin, Role: ' , self roleName].
	stdin := recordUnit data! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
receive
	self subclassResponsibility! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
requestHandler
	"Private - Answer a FSRequestHandler."

	^self server requestHandler! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
roleName
	self subclassResponsibility! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
send
	self subclassResponsibility! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
sendEndRequestSuccess
	"SPEC6.2 After sending all its stdout and stderr data, the Responder application sends a FCGI_END_REQUEST record. The application sets the protocolStatus component to FCGI_REQUEST_COMPLETE and the appStatus component to the status code that the CGI program would have returned via the exit system call."

	self connection writeRecord: (FSRecordStruct
		endRequestId: self activeId
		protocolStatus:  FCGIRequestComplete 
		appStatus: 0)! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
sendStderr
	"Private - SPEC 6.1 If an application has no errors to report, it sends either no FCGI_STDERR records or one zero-length FCGI_STDERR record"

	(stderror isNil or: [stderror isEmpty]) ifTrue: [^nil].
	self connection writeRecord: (FSRecordSeries 
		stderrId: self activeId 
		data: (stderror isNil ifTrue: [String new] ifFalse: [stderror ]))
	! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
sendStdout
	"Private - SPEC 6.1 When a role protocol calls for transmitting a stream other than FCGI_STDERR, at least one record of the stream type is always transmitted, even if the stream is empty. "

	self connection writeRecord: (FSRecordSeries 
		stdoutId: self activeId data: (stdout isNil ifTrue: [String new] ifFalse: [stdout]))
	! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
server
	"Private - Answer a FSServer."

	^connection server! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
space
	"This method is part of a puttableStream protocol for stdout"
	
	self nextPutAll: (ByteArray with: 32)
	! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
stderr: aStringOrByteArray
	"Set string to be returned to the WS by FCGI_STDERR.
	SPEC 6.1 All role protocols use the FCGI_STDERR stream just the way stderr is used in conventional applications programming: to report application-level errors in an intelligible way. Use of the FCGI_STDERR stream is always optional
	SW: There is no (smalltalk) stream access, because this is not streamed back to the server, but sent once #answer returns"


	stderror := aStringOrByteArray! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
stdin
	"Answer a ByteArray, the bytes of stdin data that was received from the Web server over FCGI_STDIN"

	^stdin! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
stdin: aByteArray params: aDictionary
	"Private - SW: Test helper method"

	params := aDictionary.
	stdin := aByteArray.! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
stdout
	^stdout! !

!FSRole methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
stdout: aStringOrByteArray
	"Set string to be returned to the WS by FCGI_STDOUT.
	SW: See #nextPutAllStdout: for alternative"

	stdout := aStringOrByteArray! !

Object subclass: #FSSocket
	instanceVariableNames: 'socketStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Protocol'!

!FSSocket class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
on: aSocket
	^ self new initialize: aSocket! !

!FSSocket methodsFor: 'other' stamp: 'AndreyLarionov 10/8/2009 03:37'!
close
	"If streaming on socket, close the socket
	-can signal errors on socket close problems"
	socketStream close.
	socketStream := nil! !

!FSSocket methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
flush! !

!FSSocket methodsFor: 'other' stamp: 'AndreyLarionov 10/8/2009 03:55'!
initialize: aSocket 
	socketStream := (SocketStream on: aSocket) binary.! !

!FSSocket methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
next
	"Answer the next byte.
	(If streaming on a socket, block if not available, and signal error if socket not valid)"

	^(self next: 1) at: 1.! !

!FSSocket methodsFor: 'other' stamp: 'DataCurator 05/16/2007 16:40'!
next2
	"Answer a two-byte unsigned integer.
	(If streaming on a socket, block if not available, and signal error if socket not valid)"

	| ba b1 b0 |
	ba := self next: 2.
	b1 := ba at: 1.
	b0 := ba at: 2.
	^b1 * 256 + b0! !

!FSSocket methodsFor: 'other' stamp: 'AndreyLarionov 10/8/2009 04:01'!
next: aCount 
	"Answer a ByteArray.
	(If streaming on a socket, block if not available, and signal error if socket not valid)"
	aCount = 0 ifTrue: [ ^ ByteArray new ].
	^socketStream next: aCount.
! !

!FSSocket methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
nextPut2: aWord
	"(If streaming on a socket signal error if socket not valid)"

	^self 
		nextPut: aWord // 256; 
		nextPut: aWord \\ 256! !

!FSSocket methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
nextPut: aByte
	"(If streaming on a socket signal error if socket not valid)"

	self nextPutAll: (ByteArray with: aByte).! !

!FSSocket methodsFor: 'other' stamp: 'AndreyLarionov 10/8/2009 04:06'!
nextPutAll: aByteArray 
	"(If streaming on a socket signal error if socket not valid)"
	aByteArray isEmpty ifTrue: [ ^ self ].
	[socketStream nextPutAll: aByteArray] 
		on: Error 
		do: [:ex | self error: 'SocketError:', socketStream socket socketError asString]! !

!FSSocket methodsFor: 'other' stamp: 'dkh 07/27/2007 13:27'!
nextRecordStruct
	"Answer the next recordStruct.
	-At this point we do not know whether it is a discrete record, or part of a series
	(If streaming on a socket, block if not available, and signal error if socket not valid)"

	^FSRecordStruct readFromStream: self! !

Object subclass: #FSSocketServer
	instanceVariableNames: 'serveLoop port connections requestHandler socket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Protocol'!

!FSSocketServer class methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
new
	^super new initialize! !

!FSSocketServer class methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
onRequestHandler: aFSResquestHandler
	"Answer an instance that will have requests handled by the aFSResquestHandler"

	^self new requestHandler: aFSResquestHandler! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
accept: aServerSocket onError: operation
	"Private - Answer an instance of connectionClass (FSConnection)
	Block in this method on accept()"

	^self subclassResponsibility! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
addConnection: aConnection
	"Private - Add the connection. (keeping the reference is only necessary to force a connection close on the receiver being sent #stop.)
	-connections is shared across multiple processes"

	"Transcript
		nextPutAll: 'addConnection ' , connections size displayString;
		cr."

	connections add: aConnection.
	aConnection server: self.
	^aConnection! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
assertAcceptConnection: aFSConnection
	"Private - SPEC3.2 -
	Not implemented(unfortunately) in Apache win32.
	-Signalling an error in this method will cause the connection to be aborted and closed.
	-Should only be sent from the aFSConnection instance."! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
connectionClass
	"Private - Answer the connections class.
	FSConnection could be subclassed to provide extra functionality like logging/profiling"

	^FSConnection! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
connectionProcessPriority
	^4! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
connections
	"Private -"

	^connections! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
description
	^''! !

!FSSocketServer methodsFor: 'other' stamp: 'AndreyLarionov 10/7/2009 03:37'!
initialize
	"Private - 
	-connections can have elements added and removed from multiple processes.
	-The only functionality needed from the connections i.v. is the ability to close outstanding connections prematurely on a server stop."

	super initialize.
	connections := "SharedSet" Set new.! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
isServing
	^serveLoop notNil! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
isStarted
	^serveLoop notNil! !

!FSSocketServer methodsFor: 'other' stamp: 'dkh 09/29/2009 15:34'!
isStarting
	^serveLoop isNil and: [ socket notNil ]! !

!FSSocketServer methodsFor: 'other' stamp: 'dkh 09/29/2009 15:34'!
isStopping
	^serveLoop notNil and: [ socket isNil ]! !

!FSSocketServer methodsFor: 'other' stamp: 'AndreyLarionov 10/8/2009 02:57'!
listen: aServerSocket
	"Private - Block in the #accept:onError: method. On an connect, add the FSConnection to the connections collection and fork a processes to serve it.
	-If an error is signaled in accept, we will keep trying. (Note the non-local return)"

	| connection |
	connection := self 
		accept: aServerSocket
		onError: [:ex | 
			self notifyUnreportableError: ex.
			^self.
		].
	connection isNil ifFalse:[[[:conn |
		self addConnection: conn.
		conn serve] value: connection ]
		forkAt: self connectionProcessPriority.]! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
maximumConnections
	"SPEC4.1 answer to web server query: FCGI_MAX_CONNS (not used by Apache)"
 
	^50! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
maximumRequests
	"SPEC4.1 answer to web server query: FCGI_MAX_REQS (not used by Apache)"

	^50! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
newServerSocket
	"Answer a ServerSocket to listen on"

	self subclassResponsibility! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
notifyFSError: anException
	"Notify that an error occurred while trying to report the aString error"

	"self subclassResponsibility"

	! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
notifyUnreportableError: aString
	"Notify that an error occurred while trying to report the aString error"

	self subclassResponsibility! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
port
	^port! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
port: anInteger
	port := anInteger! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
removeConnection: aConnection
	"Remove the connection.
	-This method must not signal an error.
	-connections is shared across multiple processes"

	connections remove: aConnection ifAbsent: []! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
requestHandler
	"Answer the FSRequestHandler."

	^requestHandler! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
requestHandler: aFSRequestHandler
	requestHandler := aFSRequestHandler! !

!FSSocketServer methodsFor: 'other' stamp: 'DataCurator 05/15/2007 17:08'!
serverProcessPriority
	^5! !

!FSSocketServer methodsFor: 'other' stamp: 'AndreyLarionov 10/7/2009 22:47'!
start

	socket := self newServerSocket.
	serveLoop := [[:sock | [true] whileTrue: [self listen: sock]] value: socket ] forkAt: self serverProcessPriority.! !

!FSSocketServer methodsFor: 'other' stamp: 'AndreyLarionov 10/8/2009 01:14'!
stop
	
	socket notNil ifTrue: [
		[socket close.] on: Error do: [].
		socket := nil].
	serveLoop notNil ifTrue: [
		[serveLoop terminate.] on: Error do: []. 
		serveLoop:= nil].
	connections notNil ifTrue: [ connections copy do: [:each | each closeConnection]].! !

FSSocketServer subclass: #FSSqSocketServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Protocol'!

!FSSqSocketServer methodsFor: 'as yet unclassified' stamp: 'AndreyLarionov 10/8/2009 02:49'!
accept: aServerSocket onError: operation
	"Private - Answer an instance of connectionClass (FSConnection)
	Block in this method on accept()"

	| clientSocket | 
	clientSocket := [aServerSocket waitForAcceptFor: self acceptTimeout.] 
		on: Error 
		do: operation.
   ^clientSocket isNil
		ifTrue: [nil]
		ifFalse: [self connectionClass socket: clientSocket ]. ! !

!FSSqSocketServer methodsFor: 'as yet unclassified' stamp: 'AndreyLarionov 10/8/2009 02:31'!
acceptTimeout
	^10! !

!FSSqSocketServer methodsFor: 'as yet unclassified' stamp: 'AndreyLarionov 10/8/2009 00:53'!
connectionProcessPriority 
	^40! !

!FSSqSocketServer methodsFor: 'as yet unclassified' stamp: 'AndreyLarionov 10/7/2009 03:05'!
description
	^'Squeak FastCGI Application'! !

!FSSqSocketServer methodsFor: 'as yet unclassified' stamp: 'AndreyLarionov 10/8/2009 00:51'!
newServerSocket
	"Private - Answer a new Socket to listen on"

	| server errStr |
	server := Socket new.
	server setOption: 'SO:REUSEADDR' value: true.
	
	(server listenOn: self port backlogSize: 4 ) isNil ifTrue: [
		errStr := server socketError asString.
		server close.
		self error: errStr.
	].
	^server.! !

!FSSqSocketServer methodsFor: 'as yet unclassified' stamp: 'AndreyLarionov 10/8/2009 02:39'!
notifyFSError: anException
	Transcript print: anException; cr.! !

!FSSqSocketServer methodsFor: 'as yet unclassified' stamp: 'AndreyLarionov 10/8/2009 01:04'!
notifyUnreportableError: anException
	Transcript print: anException.! !

!FSSqSocketServer methodsFor: 'as yet unclassified' stamp: 'AndreyLarionov 10/7/2009 22:50'!
serverProcessPriority
	^40! !

Object subclass: #FSStream
	instanceVariableNames: 'readStream writeStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Protocol'!

!FSStream class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
readStream: aReadStream writeStream: aWriteStream
	^self basicNew readStream: aReadStream writeStream: aWriteStream! !

!FSStream class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
socket: aSocket
	^self readStream: aSocket readStream writeStream: aSocket writeStream! !

!FSStream class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testStreamReading: aByteObject
	"SW Test helper method.
	-aByteArray is set (and reset) into the readStream."

	^self readStream: aByteObject asByteArray readStream writeStream: ByteArray writeStream! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
atEnd
	^readStream atEnd! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
close
	writeStream notNil ifTrue: [writeStream close].
	readStream notNil ifTrue: [readStream close].
	writeStream := readStream := nil! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
contentsSpecies
	^String! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
flush
	writeStream flush! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
hasInput 
	^readStream hasInput! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
log
	^''! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
next
	"Answer a Character"

	^Character codePoint:  readStream next! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
next: anInteger
	^(self nextBytes: anInteger) asString! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
next: count into: aSequenceableCollection startingAt: startAt

self halt.
	^readStream 
		next: count
		into: aSequenceableCollection
		startingAt: startAt! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
next: size putAll: aSequenceableCollection startingAt: start
	^writeStream 
		next: size
		putAll: aSequenceableCollection
		startingAt: start! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
nextByte
	^(self nextBytes: 1) first! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
nextBytes: anInteger

	^readStream next: anInteger.! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
nextPut: aCharacter
	^self nextPutByte: aCharacter asInteger! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
nextPutAll: aByteObject
	self nextPutBytes: aByteObject asByteArray.
	^aByteObject! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
nextPutByte: anInteger
	^writeStream nextPut: anInteger ! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
nextPutBytes: aByteArray
	^writeStream nextPutAll: aByteArray! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
peek

	| next |
	(next := self peekByte) isNil ifTrue: [^nil].
	^next asCharacter.! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
peekByte
	^readStream peek! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
readStream: aReadStream writeStream: aWriteStream
	readStream := aReadStream.
	writeStream := aWriteStream.! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
resetLogStream! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
socket
	^readStream socket! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
writeStream
	^writeStream! !

!FSStream methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
writeStream: aWriteStream
	writeStream := aWriteStream! !

TestCase subclass: #FSProtocolTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Tests'!

FSProtocolTests subclass: #FSNameValuePairTest
	instanceVariableNames: 'pair'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Tests'!

!FSNameValuePairTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
assert1Or4For: length is: bytes2 
	| bytes1 |
	bytes1 := FSNameValuePair oneOrFourBytesFor: length.
	self assert: bytes1 = bytes2.
	self assert: (FSNameValuePair next1Or4From: bytes2 readStream) = length! !

!FSNameValuePairTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
assertName: aNameString value: aValueString is: aByteArray
	| stream |
	pair := FSNameValuePair readFromStream: aByteArray readStream.
	self assert: pair name = aNameString.
	self assert: pair value = aValueString.
	stream := WriteStream on: ByteArray new.
	pair writeToStream: stream.
	self assert: stream contents = aByteArray! !

!FSNameValuePairTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
test11
	self assertName: 'a' value: 'b' is: (ByteArray withAll: #(1 1 97 98))! !

!FSNameValuePairTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
test44
	| s1 s2 |
	s1 := (String new: 128) atAllPut: $a; yourself.
	s2 := (String new: 256) atAllPut: $b; yourself.
	self 
		assertName: s1
		value: s2
		is: (ByteArray withAll: #(128 0 0 128 128 0 1 0)) , s1 asByteArray , s2 asByteArray! !

!FSNameValuePairTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
test44Hack
	pair := FSNameValuePair readFromStream: (ByteArray withAll: #(128 0 0 1 128 0 0 1 97 98)) readStream.
	self assert: pair name = 'a'.
	self assert: pair value = 'b'.! !

!FSNameValuePairTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testEmpty
	self should: [FSNameValuePair readFromStream: (ByteArray withAll: #()) readStream] raise: Error! !

!FSNameValuePairTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testPut1Or4On
	self assert1Or4For: 0 is: #(0) asByteArray.
	self assert1Or4For: 1 is: #(1) asByteArray.
	self assert1Or4For: 127 is: #(127) asByteArray.
	self assert1Or4For: 128 is: #(128 0 0 128) asByteArray.
	self assert1Or4For: 129 is: #(128 0 0 129) asByteArray.
	self assert1Or4For: 255 is: #(128 0 0 255) asByteArray.
	self assert1Or4For: 256 is: #(128 0 1 0) asByteArray! !

!FSProtocolTests class methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
isAbstract
	^self name asString last = $s.! !

FSProtocolTests subclass: #FSRequestHandlerTests
	instanceVariableNames: 'role stdin params handler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Tests'!

FSRequestHandlerTests subclass: #FSDemoHandlerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Tests'!

!FSDemoHandlerTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
setUp
	super setUp.
	handler := FSDemoHandler new! !

!FSDemoHandlerTest methodsFor: 'other' stamp: 'AndreyLarionov 10/7/2009 05:14'!
testDefault

	| |
	self paramAt: 'a1' put: 'b2'.
	stdin := 'abcd'.
	self setResponderRole.
	self answerRole.
	self assert: ('Content-type:*' match: role stdout).
	self assert: ('*al*' match: role stdout).
	self assert: ('*[97 98 99 100]*' match: role stdout ).! !

!FSDemoHandlerTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testException
	self paramAt: 'QUERY_STRING' put: 'exception'.
	self setResponderRole.
	self should: [self answerRole] raise: Error! !

!FSDemoHandlerTest methodsFor: 'other' stamp: 'AndreyLarionov 10/7/2009 04:29'!
testHTTP302
	self paramAt: 'QUERY_STRING' put: 'http302'.
	self setResponderRole.
	self answerRole.
	self assert: ('Status: 302*' match: role stdout)! !

!FSDemoHandlerTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testLarge
	self paramAt: 'QUERY_STRING' put: 'large'.
	self setResponderRole.
	self answerRole! !

!FSDemoHandlerTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testNotIncluded
	self paramAt: 'QUERY_STRING' put: 'notASelector'.
	self setResponderRole.
	self shouldnt: [self answerRole] raise: Error! !

!FSDemoHandlerTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testQueryEmpty
	self paramAt: 'QUERY_STRING' put: ''.
	self setResponderRole.
	self shouldnt: [self answerRole] raise: Error! !

!FSDemoHandlerTest methodsFor: 'other' stamp: 'AndreyLarionov 10/7/2009 04:31'!
testStatus200
	self paramAt: 'QUERY_STRING' put: 'status200'.
	self setResponderRole.
	self answerRole.
	self assert: ('Status: 200 OK*' match: role stdout)! !

!FSDemoHandlerTest methodsFor: 'other' stamp: 'AndreyLarionov 10/7/2009 04:31'!
testStatus500
	self paramAt: 'QUERY_STRING' put: 'status500'.
	self setResponderRole.
	self answerRole.
	self assert: ('Status: 500*' match: role stdout)! !

!FSDemoHandlerTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testStream
	self paramAt: 'QUERY_STRING' put: 'stream'.
	self setResponderRole.
	self answerRole.
	self assert: (self writeStream contents notEmpty)! !

!FSRequestHandlerTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
answerRole
	handler answerResponderRole: role! !

!FSRequestHandlerTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
getResponderOn: aStdinString params: aDictionary
	"aDictionary's keys and values should be strings"

	| swazooStream |
	swazooStream := FSStream
		readStream: (ReadStream on: #() asByteArray) 
		writeStream: (WriteStream on: ByteArray new).
	^(FSResponderRole onConnection: swazooStream) 
		stdin: aStdinString asByteArray
		params: aDictionary! !

!FSRequestHandlerTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
paramAt: aKeyString put: aValueString
	params at: aKeyString put: aValueString! !

!FSRequestHandlerTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
setResponderRole
	role := self getResponderOn: stdin params: params! !

!FSRequestHandlerTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
setUp
	super setUp.
	stdin := ''.
	params := Dictionary new.! !

!FSRequestHandlerTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
writeStream
	^role connection writeStream! !

FSProtocolTests subclass: #FSSocketServerTest
	instanceVariableNames: 'server'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Tests'!

!FSSocketServerTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
setUp
	server := FSSocketServer new! !

!FSSocketServerTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testConnection
	| connection |
	connection := server connectionClass new.
	server addConnection: connection.
	self assert: (server connections includes: connection).
	server removeConnection: connection.
	self deny: (server connections includes: connection).
	server removeConnection: connection.
	server removeConnection: connection.

	! !

!FSSocketServerTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testQueryResults
	server maximumRequests.
	server maximumConnections.
	! !

FSProtocolTests subclass: #FSStreamingTests
	instanceVariableNames: 'extStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Tests'!

FSStreamingTests subclass: #FSConnectionTest
	instanceVariableNames: 'connection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Tests'!

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
getDefaultGetValuesPairs
	| pairs |
	(pairs := OrderedCollection new)
		add: (FSNameValuePair name: 'FCGI_MAX_CONNS' value: '50');
		add: (FSNameValuePair name: 'FCGI_MAX_REQS' value: '50');
		add: (FSNameValuePair name: 'FCGI_MPXS_CONNS' value: '0').
	^pairs! !

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
getEmptyGetValuesPairs
	| pairs |
	(pairs := OrderedCollection new)
		add: (FSNameValuePair name: 'FCGI_MAX_CONNS' value: '');
		add: (FSNameValuePair name: 'FCGI_MAX_REQS' value: '');
		add: (FSNameValuePair name: 'FCGI_MPXS_CONNS' value: '').
	^pairs! !

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
resetConnection
	self setConnectionOn: #[]! !

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
setConnectionOn: aByteArray 
	self setExternalStreamOn: aByteArray.
	connection := FSConnection new stream: extStream! !

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
setUp
	super setUp.
	self resetConnection! !

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testAssertActiveRequest
	| r1 r2 |
	r1 := self getBeginRecord.
	r2 := FSRecordStruct 
				beginRequestId: 2
				role: 1
				flags: 0.
	self setConnectionOn: r1 asByteArray.
	connection assertActiveRequest: r1.
	connection assertActiveRequest: r2.
	connection makeActive.
	connection assertActiveRequest: r1.
	self should: [connection assertActiveRequest: r2] raise: Error! !

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testCloseConnection
	| server |
	server := FSSocketServer new.
	server addConnection: connection.
	self assert: connection server == server.
	self assert: (server connections includes: connection).
	connection closeConnection.
	self deny: (server connections includes: connection)! !

!FSConnectionTest methodsFor: 'other' stamp: ''!
testHandleError
	| ex a b |
	[self error: 'This is a test'] on: Error do: [ :e | ex	:= e].
	connection handleError: ex.
	a := self writeStream contents.
	b := (FSRecordSeries stderrId: 0 data: 'This is a test') asByteArray.
	self assert: a = b.! !

!FSConnectionTest methodsFor: 'other' stamp: ''!
testHandleErrorEx
	| ex |
	[self error: 'This is a test'] on: Error do: [ :e | ex	:= e].
	connection stream: nil.
	connection handleError: ex.! !

!FSConnectionTest methodsFor: 'other' stamp: ''!
testHandleFSError
	| ex |
	[FSError signalCantMultiplex] on: Error do: [ :e | ex	:= e].
	connection server:  FSSocketServer new.
	connection handleFSError: ex.
	self assert:self writeStream contents = (FSRecordStruct 
		endRequestId: 0 
		protocolStatus:  1
		appStatus: 0) asByteArray! !

!FSConnectionTest methodsFor: 'other' stamp: ''!
testHandleFSErrorEx
	| ex |
	[FSError signalCantMultiplex ] on: FSError do: [ :e | ex	:= e].
	connection stream: nil.
	connection handleFSError: ex.! !

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testHandleManRecords
	| records pairs |
	pairs := self getEmptyGetValuesPairs.
	(records := OrderedCollection new)
		add: (FSRecordStruct getValuesPairs: pairs);
		add: self getBeginRecord.
	self setConnectionOn: (self getByteArrayOf: records).
	connection server: FSSocketServer new.
	connection makeActive.
	self assert: (connection role isKindOf: FSResponderRole).
	self assert: self writeStream contents 
				= (FSRecordStruct getValuesResultPairs: self getDefaultGetValuesPairs) asByteArray! !

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testHandleManRecordsMiddle
	| records pairs |
	pairs := self getEmptyGetValuesPairs.
	(records := OrderedCollection new)
		add: self getBeginRecord;
		add: (FSRecordStruct getValuesPairs: pairs);
		add: self getStdinRecord.
	self setConnectionOn: (self getByteArrayOf: records).
	connection server: FSSocketServer new.
	connection makeActive.
	self assert: connection nextAppRecord = self getStdinRecord.
	self assert: (connection role isKindOf: FSResponderRole).
	self assert: self writeStream contents 
				= (FSRecordStruct getValuesResultPairs: self getDefaultGetValuesPairs) asByteArray! !

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testMakeActive
	self setConnectionOn: self getBeginRecord asByteArray.
	connection makeActive.
	self assert: (connection role isKindOf: FSResponderRole).
	self assert: connection activeId = 1! !

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testMakeActive2
	self setConnectionOn: ( FSRecordStruct beginRequestId: 2 role: 2 flags: 0) asByteArray.
	connection makeActive.
	self assert: (connection role isKindOf: FSAuthorizerRole).
	self assert: connection activeId = 2.! !

!FSConnectionTest methodsFor: 'other' stamp: 'dkh 07/27/2007 13:27'!
testMakeActiveEx
	self setConnectionOn: (FSRecordStruct beginRequestId: 1 role: 1 flags: 1) asByteArray.
	self should: [connection makeActive] raise: FSError! !

!FSConnectionTest methodsFor: 'other' stamp: ''!
testNextAppUnitEx
	| r1 r2 ex |
	r1 := self getStdinRecord.
	r2 := FSRecordSeries stdinId: 2 data: 'abc'.
	self setConnectionOn: r1 asByteArray , r2 asByteArray.
	connection setActiveIdFor: r1.
	self assert: connection nextAppRecord = r1.
	[connection nextAppRecord] on: FSError do: [:e | ex := e].
	self assert: ex protocolStatus = 1	"FCGI_CANT_MPX_CONN"! !

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testResultsOfQueryPairs
	| results |
	connection server: FSSocketServer new.
	results := connection resultsOfQueryPairs: self getEmptyGetValuesPairs.
	self assert: results size = 3.
	results do: 
			[:each | 
			self assert: (each name isKindOf: String).
			self assert: (each value isKindOf: String)]! !

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testResultsOfQueryPairsEmpty
	| results |
	connection server: FSSocketServer new.
	results := connection resultsOfQueryPairs: #().
	self assert: results isEmpty! !

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testResultsOfQueryPairsSub
	| results |
	connection server: FSSocketServer new.
	results := connection resultsOfQueryPairs: (Array with: self getEmptyGetValuesPairs last).
	self assert: results size = 1.
	self assert: results first name = 'FCGI_MPXS_CONNS'.
	self assert: results first value = '0'! !

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testRoleClassFor
	self assert: (connection roleClassFor: 1) = FSResponderRole.
	self assert: (connection roleClassFor: 2) = FSAuthorizerRole.
	self should: [connection roleClassFor: 20] raise: Error .! !

!FSConnectionTest methodsFor: 'other' stamp: ''!
testRoleClassForEx
	| ex |
	[connection roleClassFor: 20] on: FSError do: [ :e | ex := e].
	self assert: ex protocolStatus = 3. "FCGI_UNKNOWN_ROLE"! !

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testSetConnectionRole
	connection setConnectionRoleFor: self getBeginRecord.
	self assert: (connection role isKindOf: FSResponderRole).
	self assert: connection role connection == connection! !

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testUnknownMananagment
	| r1 r2 bytes |
	r1 := FSRecordStruct type: 18 requestId: 0 data: ''.
	r2 := self getStdinRecord.
	bytes := r1 asByteArray , r2 asByteArray.
	self setConnectionOn: bytes.
	self assert: connection nextAppRecord = r2.
	self assert: self writeStream contents 
				= (FSRecordStruct unknownType: 18) asByteArray! !

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testUnknownMananagment2
	| r1 r2 bytes unkBytes |
	r1 := FSRecordStruct unknownType: 20.
	r2 := self getStdinRecord.
	bytes := r1 asByteArray , r1 asByteArray , r2 asByteArray.
	unkBytes := (FSRecordStruct unknownType: 11) asByteArray.
	self setConnectionOn: bytes.
	self assert: connection nextAppRecord = r2.
	self assert: self writeStream contents = (unkBytes , unkBytes)! !

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testWriteRecord
	| record |
	record := self getEndRequestRecord.
	connection writeRecord: record.
	self assert: self writeStream contents = record asByteArray! !

!FSConnectionTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testWriteRecordSeries
	| record |
	record := self getStdoutRecord.
	connection writeRecord: record.
	self assert: self writeStream contents = record asByteArray! !

FSStreamingTests subclass: #FSRecordSeriesTest
	instanceVariableNames: 'recordSeries'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Tests'!

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
assertConstructionCommon
	self assert: recordSeries records notEmpty.
	self assert: recordSeries requestId = 1.
	self assert: recordSeries lastRecord isLastInSeries.
	recordSeries records do: 
			[:each | 
			self assert: each requestId = 1.
			self assert: (each isKindOf: FSRecordStruct)]! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
setRecordSeriesOn: aByteArray 
	self setExternalStreamOn: aByteArray.
	recordSeries := FSRecordSeries readFromStream: extStream! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
setSingleNonEmpty
	self setRecordSeriesOn: #(1 5 0 1 0 5 0 0 1 2 3 4 5 1 5 0 1 0 0 0 0 100) asByteArray! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
setZeroNonEmpty
	self setRecordSeriesOn: #(1 5 0 1 0 0 0 0 100) asByteArray! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
streamAndAssertContents: aByteArray 
	self setEmptyExternalStream.
	recordSeries streamToStream: extStream.
	self assertBytesWritten: aByteArray! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testAsParamsEmpty
	| params |
	self setRecordSeriesOn: #(1 4 0 1 0 0 0 0 ) asByteArray.
	params := recordSeries asParams.
	self assert: params isEmpty! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testAsParamsMultiple
	| params |
	self setRecordSeriesOn: #(1 4 0 1 0 8 0 0 1 1 97 98 1 1 99 100 1 4 0 1 0 0 0 0) asByteArray.
	params := recordSeries asParams.
	self assert: (params at: 'a') = 'b'.
	self assert: (params at: 'c') = 'd'.
	self assert: params size = 2
	! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testAsParamsSingle
	| params |
	self setRecordSeriesOn: #(1 4 0 1 0 4 0 0 1 1 97 98 1 4 0 1 0 0 0 0) asByteArray.
	params := recordSeries asParams.
	self assert: (params at: 'a') = 'b'.
	self assert: params size = 1! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testAssertValidEx
	self setZeroNonEmpty.
	recordSeries records removeAll: recordSeries records copy.
	self should: [recordSeries assertValid] raise: Error! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'dkh 11/27/2007 16:29'!
testConstructionNewOutEmpty
	recordSeries := FSRecordSeries stdoutId: 1 data: ''.
	self assertConstructionCommon.
	self assert: recordSeries records size = 2. "Not sure why this works ... from reading code, this appears to always have been true - dkh"
	self assert: recordSeries requestId = 1.
	self assert: recordSeries firstRecord isLastInSeries.
	self assert: recordSeries firstRecord data isEmpty! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testConstructionNewOutMulti65536

	recordSeries := FSRecordSeries stdoutId: 1
				data: ((String new: 65534) atAllPut: $a; yourself) , 'b' , 'c'.
	self assertConstructionCommon.
	self assert: recordSeries records size = 3.
	self assert: (recordSeries records at: 3) isLastInSeries.
	self assert: recordSeries firstRecord data 
				= (((String new: 65534) atAllPut: $a; yourself) , 'b') asByteArray.
	self assert: (recordSeries records at: 2) data = 'c' asByteArray.
	self assert: recordSeries lastRecord data isEmpty! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testConstructionNewOutSingle
	recordSeries := self getStdoutRecord.
	self assertConstructionCommon.
	self assert: recordSeries records size = 2.
	self assert: (recordSeries records at: 2) isLastInSeries.
	self assert: recordSeries firstRecord data = 'abc' asByteArray.
	self assert: recordSeries lastRecord data isEmpty! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testConstructionNewOutSingle65535
	| data |
	data := ((String new: 65534) atAllPut: $a; yourself) , 'b'.
	recordSeries := FSRecordSeries stdoutId: 1 data: data.
	self assertConstructionCommon.
	self assert: recordSeries records size = 2.
	self assert: (recordSeries records at: 2) isLastInSeries.
	self assert: recordSeries firstRecord data = data asByteArray.
	self assert: recordSeries lastRecord data isEmpty! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testFirstRecord
	self setZeroNonEmpty.
	self assert: recordSeries firstRecord == recordSeries records first.! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testSingleNonEmpty
	self setSingleNonEmpty.
	self assert: (recordSeries isKindOf: FSRecordSeries).
	self assert: extStream next = 100.
	self assert: recordSeries records size = 2.
	self assert: recordSeries data = #(1 2 3 4 5) asByteArray.
	self assert: recordSeries requestId = 1.
	recordSeries assertValid! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testStreamEmpty
	recordSeries := FSRecordSeries stdoutId: 1 data: ''.
	self writeAndAssertContents: #(1 6 0 1 0 0 0 0) asByteArray! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testStreamStdout
	recordSeries := self getStdoutRecord.
	self streamAndAssertContents: #(1 6 0 1 0 3 0 0 97 98 99) asByteArray! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testStreamStdout2a
	| string |
	string := (String new: 65535) atAllPut: $a; yourself.
	recordSeries := recordSeries := FSRecordSeries stdoutId: 1 data:  string .

	self streamAndAssertContents: #(1 6 0 1 255 255 0 0 ) asByteArray , string asByteArray ! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testStreamStdout2b
	| string |
	string := (String new: 65535) atAllPut: $a; yourself.
	recordSeries := recordSeries := FSRecordSeries stdoutId: 1 data:  string , 'b'.

	self streamAndAssertContents: 
		#(1 6 0 1 255 255 0 0) asByteArray , 
		string asByteArray , 
		#(1 6 0 1 0 1 0 0 98) asByteArray! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testWriteEmpty
	recordSeries := FSRecordSeries stdoutId: 1 data: ''.
	self writeAndAssertContents: #(1 6 0 1 0 0 0 0) asByteArray! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testWriteStderr
	recordSeries := (FSRecordSeries stderrId: 1 data: 'abc').
	self writeAndAssertContents: #(1 7 0 1 0 3 0 0 97 98 99 1 7 0 1 0 0 0 0) asByteArray! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testWriteStdout
	recordSeries := self getStdoutRecord.
	self writeAndAssertContents: #(1 6 0 1 0 3 0 0 97 98 99 1 6 0 1 0 0 0 0) asByteArray! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testWriteStdout2a
	| string |
	string := (String new: 65535) atAllPut: $a; yourself.
	recordSeries := recordSeries := FSRecordSeries stdoutId: 1 data:  string .

	self streamAndAssertContents: 
		#(1 6 0 1 255 255 0 0) asByteArray , 
		string asByteArray , 
		#(1 6 0 1 0 0 0 0) asByteArray! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testWriteStdout2b
	| string |
	string := (String new: 65535) atAllPut: $a; yourself.
	recordSeries := recordSeries := FSRecordSeries stdoutId: 1 data:  string , 'b'.

	self streamAndAssertContents: 
		#(1 6 0 1 255 255 0 0) asByteArray , 
		string asByteArray , 
		#(1 6 0 1 0 1 0 0 98) asByteArray , 
		#(1 6 0 1 0 0 0 0) asByteArray! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testZeroNonEmpty
	self setZeroNonEmpty.
	self assert: (recordSeries isKindOf: FSRecordSeries).
	self assert: extStream next = 100.
	self assert: recordSeries records size = 1.
	self assert: recordSeries data isEmpty.
	self assert: recordSeries requestId = 1.
	recordSeries assertValid! !

!FSRecordSeriesTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
writeAndAssertContents: aByteArray 
	self setEmptyExternalStream.
	recordSeries writeToStream: extStream.
	self assertBytesWritten: aByteArray! !

FSStreamingTests subclass: #FSRecordStructTest
	instanceVariableNames: 'record'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Tests'!

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
assertCommon
	self assert: (record isKindOf: FSRecordStruct).
	self assert: record requestId = 1! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
setRecordOn: aByteArray 
	self setExternalStreamOn: aByteArray.
	record := FSRecordStruct readFromStream: extStream! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testAsPairs
	record := FSRecordStruct getValuesPairs: (Array 
		with: (FSNameValuePair name: 'a' value: 'b')
		with: (FSNameValuePair name: 'c' value: 'd')).
	self writeAndAssertContents: #(1 9 0 0 0 8 0 0 1 1 97 98 1 1 99 100) asByteArray.! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testIsManagment
	self assert: (FSRecordStruct type: 1 requestId: 0 data: '') isManagement.
	self deny: self getBeginRecord isManagement.
	self assert: (FSRecordStruct type: 4 requestId: 0 data: '') isManagement.
	self deny: (FSRecordStruct type: 4 requestId: 1 data: '') isManagement.! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testReadBeginRequest
	self setRecordOn: #(1 1 0 1 0 8 0 0 1 2 3 4 5 6 7 8 9) asByteArray.
	self assert: extStream next = 9.
	self assertCommon.
	self assert: record data = #(1 2 3 4 5 6 7 8) asByteArray.
	self assert: record role = (1 * 256 + 2).
	self assert: record flags = 3.
	self assert: record wantsKeepConnection! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testReadBeginRequestKeepConnNot
	self setRecordOn: #(1 1 0 1 0 8 0 0 1 2 0 4 5 6 7 8 9) asByteArray.
	self assert: extStream next = 9.
	self assertCommon.
	self deny: record wantsKeepConnection! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testReadData
	self setRecordOn: #(1 4 0 1 0 5 0 0 1 2 3 4 5 6 7 8 9) asByteArray.
	self assert: extStream next = 6.
	self assertCommon.
	self assert: record data = #(1 2 3 4 5) asByteArray! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testReadEmpty
	self setRecordOn: #(1 5 0 1 0 0 0 0 0 1 0) asByteArray.
	self assertCommon.
	self assert: record contentLength = 0.
	self assert: record type = 5.
	self assert: record data isEmpty.
	self assert: record isLastInSeries.
	self assert: extStream next = 0.
	self assert: extStream next = 1! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testReadExInvalidContentLength
	self should: [self setRecordOn: #(1 1 0 1 0 6 0 0 1 2 3 4 5 6 7 8 9) asByteArray]
		raise: Error! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testReadExVersion
	self should: [self setRecordOn: #(2 1 0 1 0 8 0 0 1 2 3 4 5 6 7 8 9) asByteArray]
		raise: Error! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testReadHigh
	| dataBytes |
	dataBytes := (ByteArray new: (5 * 256 + 8)) atAllPut: 1; yourself.
	self setRecordOn: #(1 2 1 2 5 8 0 0) asByteArray , dataBytes  , (ByteArray with: 100).
	self assert: record requestId = (1 * 256 + 2).
	self assert: record contentLength = (5 * 256 + 8).
	self assert: extStream next = 100.
	self assert: record data = dataBytes ! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testReadPadded
	self setRecordOn: #(1 5 0 1 0 5 2 0 1 2 3 4 5 6 7 8 9) asByteArray.
	self assert: extStream next = 8.
	self assertCommon.
	self assert: record data = #(1 2 3 4 5) asByteArray! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testWrite
	record := FSRecordStruct 
				type: 6
				requestId: 1
				data: #(1 2) asByteArray.
	self writeAndAssertContents: #(1 6 0 1 0 2 0 0 1 2) asByteArray.
	record := FSRecordStruct 
				type: 6
				requestId: 1
				data: 'abc'.
	self writeAndAssertContents: #(1 6 0 1 0 3 0 0 97 98 99) asByteArray.
	record := FSRecordStruct 
				type: 5
				requestId: 2
				data: #(1 2) asByteArray.
	self writeAndAssertContents: #(1 5 0 2 0 2 0 0 1 2) asByteArray! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testWriteEndRequest
	record := FSRecordStruct endRequestId: 2 protocolStatus: 0 appStatus: 0.
	self writeAndAssertContents: #(1 3 0 2 0 8 0 0 0 0 0 0 0 0 0 0) asByteArray! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testWriteEndRequest2
	record := FSRecordStruct endRequestId: 1 protocolStatus:  0 appStatus: 0.
	self writeAndAssertContents: #(1 3 0 1 0 8 0 0 0 0 0 0 0 0 0 0) asByteArray.
	record := FSRecordStruct endRequestId: 1 protocolStatus:  2 appStatus: 0.
	self writeAndAssertContents: #(1 3 0 1 0 8 0 0 0 0 0 0 2 0 0 0) asByteArray.
	record := FSRecordStruct endRequestId: 1 protocolStatus:  0 appStatus: 1.
	self writeAndAssertContents: #(1 3 0 1 0 8 0 0 0 0 0 1 0 0 0 0) asByteArray.! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testWriteGetValuesPairs
	record := FSRecordStruct getValuesPairs: (Array 
		with: (FSNameValuePair name: 'a' value: 'b')
		with: (FSNameValuePair name: 'c' value: 'd')).
	self writeAndAssertContents: #(1 9 0 0 0 8 0 0 1 1 97 98 1 1 99 100) asByteArray.! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testWriteGetValuesResultPairs
	record := FSRecordStruct getValuesResultPairs: (Array 
		with: (FSNameValuePair name: 'a' value: 'b')
		with: (FSNameValuePair name: 'c' value: 'd')).
	self writeAndAssertContents: #(1 10 0 0 0 8 0 0 1 1 97 98 1 1 99 100) asByteArray.! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testWriteGetValuesResultPairsEmpty
	record := FSRecordStruct getValuesResultPairs: #().
	self writeAndAssertContents: #(1 10 0 0 0 0 0 0 ) asByteArray.! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testWriteStdOut
	"?"
	record := FSRecordStruct type: 6 requestId: 2 data: ''.
	self writeAndAssertContents: #(1 6 0 2 0 0 0 0) asByteArray.
	record := FSRecordStruct type: 6 requestId: 2 data: 'abc'.
	self writeAndAssertContents: #(1 6 0 2 0 3 0 0 97 98 99) asByteArray! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testWriteUnknownType
	record := FSRecordStruct unknownType: 2.
	self writeAndAssertContents: #(1 11 0 0 0 8 0 0 2 0 0 0 0 0 0 0) asByteArray.
	record := FSRecordStruct unknownType: 14.
	self writeAndAssertContents: #(1 11 0 0 0 8 0 0 14 0 0 0 0 0 0 0) asByteArray! !

!FSRecordStructTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
writeAndAssertContents: aByteArray 
	self setEmptyExternalStream.
	record writeToStream: extStream.
	self assertBytesWritten: aByteArray! !

FSStreamingTests subclass: #FSRoleTests
	instanceVariableNames: 'role'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Tests'!

FSRoleTests subclass: #FSAuthorizerRoleTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Tests'!

!FSAuthorizerRoleTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
assertRoleClass
	self assert: (role isKindOf: FSAuthorizerRole)! !

!FSAuthorizerRoleTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
roleNum
	^2! !

!FSAuthorizerRoleTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testConstruction
	self setRoleOn: self getBeginRecord asByteArray.
	self assert: role connection role == role.
	self assert: role roleName = 'Authorizer'! !

FSRoleTests subclass: #FSResponderRoleTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FastCGI-Tests'!

!FSResponderRoleTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
assertRoleClass
	self assert: (role isKindOf: FSResponderRole)! !

!FSResponderRoleTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
roleNum
	^1! !

!FSResponderRoleTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testConstruction
	self setRoleOn: self getBeginRecord asByteArray.
	self assert: (role isKindOf: FSResponderRole).
	self assert: role connection role == role.
	self assert: role roleName = 'Responder'! !

!FSResponderRoleTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testContentLengthHeader
	role := FSResponderRole new.
	role stdin: '' params: (Dictionary new).
	self assert: role contentLengthHeader = 0.
	role params at: 'CONTENT_LENGTH' put: '30'.
	self assert: role contentLengthHeader = 30.! !

!FSResponderRoleTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testEmptyStdin
	self setRoleOnRecords: (self getSampleRecordsWith: self getStdinRecordEmpty).
	self assert: (role isKindOf: FSResponderRole).
	self assert: role connection role == role.
	role connection server requestHandler: FSDemoHandler new.
	role receive.
	self assert: role stdin isEmpty.
	self assert: (role params at: 'a') = 'b'.
	role answer.
	role send! !

!FSResponderRoleTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testReceiveStdinEx
	self setRoleOnRecords: (self getSampleRecordsWith: (FSRecordStruct type: 20 requestId: 1 data: '')).
	self should: [role receive] raise: Error
	! !

!FSResponderRoleTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testReceiveStdinLengthEx
	self 
		setRoleOnRecords: (self getSampleRecordsWith: self getStdinRecord).
	role readParams.
	role readStdin.
	self should: [role assertStdinLength] raise: Error.
	self 
		setRoleOnRecords: (self getSampleRecordsWith: self getStdinRecord).
	role readParams.
	role params at: 'CONTENT_LENGTH' put: '3'.
	role readStdin.
	role assertStdinLength.
	self assert: role stdin = 'abc' asByteArray! !

!FSResponderRoleTest methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testStreamStdout
	self setDefaultEmptyRole.
	role nextPutAll: 'abc'.
	self assertBytesWritten: #(1 6 0 1 0 3 0 0 97 98 99) asByteArray.
	role nextPutAll: 'def'.
	self assertBytesWritten: #(1 6 0 1 0 3 0 0 97 98 99) asByteArray , #(1 6 0 1 0 3 0 0 100 101 102) asByteArray.
	role nextPutAll: ''.
	self assertBytesWritten: #(1 6 0 1 0 3 0 0 97 98 99) asByteArray , #(1 6 0 1 0 3 0 0 100 101 102) asByteArray.! !

!FSRoleTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
getBeginRecord
	^FSRecordStruct 
		beginRequestId: 1
		role: self roleNum
		flags: 0! !

!FSRoleTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
getSampleRecords
	^Array with: self getBeginRecord
		with: (FSRecordSeries paramsId: 1
				pairs: (Array 
						with: (FSNameValuePair name: 'a' value: 'b')
						with: (FSNameValuePair name: 'c' value: 'd')
						with: (FSNameValuePair name: 'CONTENT_LENGTH' value: '0')))! !

!FSRoleTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
getSampleRecordsWith: aRecord
	^self getSampleRecords , (Array with: aRecord)
	! !

!FSRoleTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
roleNum 
	self subclassResponsibility! !

!FSRoleTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
setDefaultEmptyRole
	self setRoleOn: (FSRecordStruct beginRequestId: 1 role: self roleNum flags: 0) asByteArray.
	self assertRoleClass
	! !

!FSRoleTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
setRoleOn: aByteArray 
	| connection |
	self setExternalStreamOn: aByteArray.
	connection := FSConnection new stream: extStream.
	connection makeActive.
	connection server: FSSocketServer new.
	role := connection role.
	self assertRoleClass! !

!FSRoleTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
setRoleOnRecords: aCollOfRecords
	self setRoleOn: (self getByteArrayOf: aCollOfRecords)
	! !

!FSRoleTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testReceivePararms
	self setRoleOnRecords: (self getSampleRecordsWith: self getStdinRecordEmpty).
	role readParams.
	self assert: (role params size = 3).
	self assert: (role params at: 'a') = 'b'.
	self assert: (role params at: 'c') = 'd'.
	self assert: (role params at: 'CONTENT_LENGTH') = '0'.! !

!FSRoleTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testReceivePararmsEx
	self setRoleOnRecords: (Array with: self getBeginRecord
				with: self getStdinRecordEmpty).
	self should: [role receive] raise: Error! !

!FSRoleTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testSendEndSuccess
	self setDefaultEmptyRole.
	role sendEndRequestSuccess.
	self assertBytesWritten: #(1 3 0 1 0 8 0 0 0 0 0 0 0 0 0 0) asByteArray! !

!FSRoleTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testSendStderr
	self setDefaultEmptyRole.
	role stderr: 'abc'.
	role sendStderr.
	self assertBytesWritten: #(1 7 0 1 0 3 0 0 97 98 99 1 7 0 1 0 0 0 0) asByteArray! !

!FSRoleTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testSendStderrEmpty
	self setDefaultEmptyRole.
	role sendStderr.
	self assert: self writeStream contents isEmpty! !

!FSRoleTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testSendStdout
	self setDefaultEmptyRole.
	role stdout: 'abc'.
	role sendStdout.
	self assertBytesWritten: #(1 6 0 1 0 3 0 0 97 98 99 1 6 0 1 0 0 0 0) asByteArray! !

!FSRoleTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testSendStdoutBytes
	self setDefaultEmptyRole.
	role stdout: 'abc' asByteArray.
	role sendStdout.
	self assertBytesWritten: #(1 6 0 1 0 3 0 0 97 98 99 1 6 0 1 0 0 0 0) asByteArray! !

!FSRoleTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testSendStdoutEmpty
	self setDefaultEmptyRole.
	role stdout: ''.
	role sendStdout.
	self assertBytesWritten: #(1 6 0 1 0 0 0 0) asByteArray! !

!FSRoleTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testSendStdoutNil
	self setDefaultEmptyRole.
	role stdout: nil.
	role sendStdout.
	self assertBytesWritten: #(1 6 0 1 0 0 0 0) asByteArray! !

!FSRoleTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
testUnknownTyped
	self 
		should: [self setRoleOnRecords: (self getSampleRecordsWith: (FSRecordSeries type: 20 requestId: 1 data: ''))]
		raise: Error.! !

!FSStreamingTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
assertBytesWritten: aByteArray
	self writeStream contents = aByteArray! !

!FSStreamingTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
getBeginRecord
	^FSRecordStruct 
		beginRequestId: 1
		role: 1
		flags: 0! !

!FSStreamingTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
getByteArrayOf: aCollOfRecords
	| stream |
	stream := WriteStream on: ByteArray new.
	aCollOfRecords do: [ :each |  stream nextPutAll: each asByteArray].
	^stream contents! !

!FSStreamingTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
getEndRequestRecord
	^FSRecordStruct 
		endRequestId: 1
		protocolStatus: 0
		appStatus: 0! !

!FSStreamingTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
getStdinRecord
	^FSRecordSeries stdinId: 1 data: 'abc'! !

!FSStreamingTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
getStdinRecordEmpty
	^FSRecordSeries stdinId: 1 data: ''! !

!FSStreamingTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
getStdoutRecord
	^FSRecordSeries stdoutId: 1 data: 'abc'! !

!FSStreamingTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
setEmptyExternalStream
	self setExternalStreamOn: #[]! !

!FSStreamingTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
setExternalStreamOn: aByteArray 
	"This method sets a Stream which has the same protocol (or is the same) as  FSStream or FasTalk's FCGIExternalStream, but on a ReadWriteByteStream.
	-aByteArray is set (and reset) into the readStream.
	-The contents of the writeStream can be accessed through (FSStreamingTests>>writeStream).
	-A Method that needs similar attention is FSStream>>convertToByteArray: "

	extStream := FSFCGIStream testStreamOn: aByteArray ! !

!FSStreamingTests methodsFor: 'other' stamp: 'DataCurator 04/28/2007 12:01'!
writeStream
	^extStream stream writeStream! !
FCGIConstants initialize!
